"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    service_name: str,
    cluster: str = None,
    task_definition: str = None,
    load_balancers: List = None,
    service_registries: List = None,
    desired_count: int = None,
    client_token: str = None,
    launch_type: str = None,
    capacity_provider_strategy: List = None,
    platform_version: str = None,
    role: str = None,
    deployment_configuration: Dict = None,
    placement_constraints: List = None,
    placement_strategy: List = None,
    network_configuration: Dict = None,
    health_check_grace_period_seconds: int = None,
    scheduling_strategy: str = None,
    deployment_controller: Dict = None,
    tags: List = None,
    enable_ecs_managed_tags: bool = None,
    propagate_tags: str = None,
    enable_execute_command: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Runs and maintains a desired number of tasks from a specified task definition. If the number of tasks running in a service drops below the desiredCount, Amazon ECS runs another copy of the task in the specified cluster. To update an existing service, see the UpdateService action. In addition to maintaining the desired count of tasks in your service, you can optionally run your service behind one or more load balancers. The load balancers distribute traffic across the tasks that are associated with the service. For more information, see Service Load Balancing in the Amazon Elastic Container Service Developer Guide. Tasks for services that do not use a load balancer are considered healthy if they're in the RUNNING state. Tasks for services that do use a load balancer are considered healthy if they're in the RUNNING state and the container instance that they're hosted on is reported as healthy by the load balancer. There are two service scheduler strategies available:    REPLICA - The replica scheduling strategy places and maintains the desired number of tasks across your cluster. By default, the service scheduler spreads tasks across Availability Zones. You can use task placement strategies and constraints to customize task placement decisions. For more information, see Service Scheduler Concepts in the Amazon Elastic Container Service Developer Guide.    DAEMON - The daemon scheduling strategy deploys exactly one task on each active container instance that meets all of the task placement constraints that you specify in your cluster. The service scheduler also evaluates the task placement constraints for running tasks and will stop tasks that do not meet the placement constraints. When using this strategy, you don't need to specify a desired number of tasks, a task placement strategy, or use Service Auto Scaling policies. For more information, see Service Scheduler Concepts in the Amazon Elastic Container Service Developer Guide.   You can optionally specify a deployment configuration for your service. The deployment is triggered by changing properties, such as the task definition or the desired count of a service, with an UpdateService operation. The default value for a replica service for minimumHealthyPercent is 100%. The default value for a daemon service for minimumHealthyPercent is 0%. If a service is using the ECS deployment controller, the minimum healthy percent represents a lower limit on the number of tasks in a service that must remain in the RUNNING state during a deployment, as a percentage of the desired number of tasks (rounded up to the nearest integer), and while any container instances are in the DRAINING state if the service contains tasks using the EC2 launch type. This parameter enables you to deploy without using additional cluster capacity. For example, if your service has a desired number of four tasks and a minimum healthy percent of 50%, the scheduler might stop two existing tasks to free up cluster capacity before starting two new tasks. Tasks for services that do not use a load balancer are considered healthy if they're in the RUNNING state. Tasks for services that do use a load balancer are considered healthy if they're in the RUNNING state and they're reported as healthy by the load balancer. The default value for minimum healthy percent is 100%. If a service is using the ECS deployment controller, the maximum percent parameter represents an upper limit on the number of tasks in a service that are allowed in the RUNNING or PENDING state during a deployment, as a percentage of the desired number of tasks (rounded down to the nearest integer), and while any container instances are in the DRAINING state if the service contains tasks using the EC2 launch type. This parameter enables you to define the deployment batch size. For example, if your service has a desired number of four tasks and a maximum percent value of 200%, the scheduler may start four new tasks before stopping the four older tasks (provided that the cluster resources required to do this are available). The default value for maximum percent is 200%. If a service is using either the CODE_DEPLOY or EXTERNAL deployment controller types and tasks that use the EC2 launch type, the minimum healthy percent and maximum percent values are used only to define the lower and upper limit on the number of the tasks in the service that remain in the RUNNING state while the container instances are in the DRAINING state. If the tasks in the service use the Fargate launch type, the minimum healthy percent and maximum percent values aren't used, although they're currently visible when describing your service. When creating a service that uses the EXTERNAL deployment controller, you can specify only parameters that aren't controlled at the task set level. The only required parameter is the service name. You control your services using the CreateTaskSet operation. For more information, see Amazon ECS Deployment Types in the Amazon Elastic Container Service Developer Guide. When the service scheduler launches new tasks, it determines task placement in your cluster using the following logic:   Determine which of the container instances in your cluster can support your service's task definition (for example, they have the required CPU, memory, ports, and container instance attributes).   By default, the service scheduler attempts to balance tasks across Availability Zones in this manner (although you can choose a different placement strategy) with the placementStrategy parameter):   Sort the valid container instances, giving priority to instances that have the fewest number of running tasks for this service in their respective Availability Zone. For example, if zone A has one running service task and zones B and C each have zero, valid container instances in either zone B or C are considered optimal for placement.   Place the new service task on a valid container instance in an optimal Availability Zone (based on the previous steps), favoring container instances with the fewest number of running tasks for this service.

    Args:
        cluster(str, optional): The short name or full Amazon Resource Name (ARN) of the cluster on which to run your service. If you do not specify a cluster, the default cluster is assumed. Defaults to None.
        service_name(str): The name of your service. Up to 255 letters (uppercase and lowercase), numbers, underscores, and hyphens are allowed. Service names must be unique within a cluster, but you can have similarly named services in multiple clusters within a Region or across multiple Regions.
        task_definition(str, optional): The family and revision (family:revision) or full ARN of the task definition to run in your service. If a revision is not specified, the latest ACTIVE revision is used. A task definition must be specified if the service is using either the ECS or CODE_DEPLOY deployment controllers. Defaults to None.
        load_balancers(List, optional): A load balancer object representing the load balancers to use with your service. For more information, see Service Load Balancing in the Amazon Elastic Container Service Developer Guide. If the service is using the rolling update (ECS) deployment controller and using either an Application Load Balancer or Network Load Balancer, you must specify one or more target group ARNs to attach to the service. The service-linked role is required for services that make use of multiple target groups. For more information, see Using service-linked roles for Amazon ECS in the Amazon Elastic Container Service Developer Guide. If the service is using the CODE_DEPLOY deployment controller, the service is required to use either an Application Load Balancer or Network Load Balancer. When creating an CodeDeploy deployment group, you specify two target groups (referred to as a targetGroupPair). During a deployment, CodeDeploy determines which task set in your service has the status PRIMARY and associates one target group with it, and then associates the other target group with the replacement task set. The load balancer can also have up to two listeners: a required listener for production traffic and an optional listener that allows you perform validation tests with Lambda functions before routing production traffic to it. After you create a service using the ECS deployment controller, the load balancer name or target group ARN, container name, and container port specified in the service definition are immutable. If you are using the CODE_DEPLOY deployment controller, these values can be changed when updating the service. For Application Load Balancers and Network Load Balancers, this object must contain the load balancer target group ARN, the container name (as it appears in a container definition), and the container port to access from the load balancer. The load balancer name parameter must be omitted. When a task from this service is placed on a container instance, the container instance and port combination is registered as a target in the target group specified here. For Classic Load Balancers, this object must contain the load balancer name, the container name (as it appears in a container definition), and the container port to access from the load balancer. The target group ARN parameter must be omitted. When a task from this service is placed on a container instance, the container instance is registered with the load balancer specified here. Services with tasks that use the awsvpc network mode (for example, those with the Fargate launch type) only support Application Load Balancers and Network Load Balancers. Classic Load Balancers are not supported. Also, when you create any target groups for these services, you must choose ip as the target type, not instance, because tasks that use the awsvpc network mode are associated with an elastic network interface, not an Amazon EC2 instance. Defaults to None.
        service_registries(List, optional): The details of the service discovery registry to associate with this service. For more information, see Service discovery.  Each service may be associated with one service registry. Multiple service registries per service isn't supported. . Defaults to None.
        desired_count(int, optional): The number of instantiations of the specified task definition to place and keep running on your cluster. This is required if schedulingStrategy is REPLICA or is not specified. If schedulingStrategy is DAEMON then this is not required. Defaults to None.
        client_token(str, optional): Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. Up to 32 ASCII characters are allowed. Defaults to None.
        launch_type(str, optional): The infrastructure on which to run your service. For more information, see Amazon ECS launch types in the Amazon Elastic Container Service Developer Guide. The FARGATE launch type runs your tasks on Fargate On-Demand infrastructure.  Fargate Spot infrastructure is available for use but a capacity provider strategy must be used. For more information, see Fargate capacity providers in the Amazon ECS User Guide for Fargate.  The EC2 launch type runs your tasks on Amazon EC2 instances registered to your cluster. The EXTERNAL launch type runs your tasks on your on-premise server or virtual machine (VM) capacity registered to your cluster. A service can use either a launch type or a capacity provider strategy. If a launchType is specified, the capacityProviderStrategy parameter must be omitted. Defaults to None.
        capacity_provider_strategy(List, optional): The capacity provider strategy to use for the service. If a capacityProviderStrategy is specified, the launchType parameter must be omitted. If no capacityProviderStrategy or launchType is specified, the defaultCapacityProviderStrategy for the cluster is used. Defaults to None.
        platform_version(str, optional): The platform version that your tasks in the service are running on. A platform version is specified only for tasks using the Fargate launch type. If one isn't specified, the LATEST platform version is used by default. For more information, see Fargate platform versions in the Amazon Elastic Container Service Developer Guide. Defaults to None.
        role(str, optional): The name or full Amazon Resource Name (ARN) of the IAM role that allows Amazon ECS to make calls to your load balancer on your behalf. This parameter is only permitted if you are using a load balancer with your service and your task definition does not use the awsvpc network mode. If you specify the role parameter, you must also specify a load balancer object with the loadBalancers parameter.  If your account has already created the Amazon ECS service-linked role, that role is used by default for your service unless you specify a role here. The service-linked role is required if your task definition uses the awsvpc network mode or if the service is configured to use service discovery, an external deployment controller, multiple target groups, or Elastic Inference accelerators in which case you should not specify a role here. For more information, see Using service-linked roles for Amazon ECS in the Amazon Elastic Container Service Developer Guide.  If your specified role has a path other than /, then you must either specify the full role ARN (this is recommended) or prefix the role name with the path. For example, if a role with the name bar has a path of /foo/ then you would specify /foo/bar as the role name. For more information, see Friendly names and paths in the IAM User Guide. Defaults to None.
        deployment_configuration(Dict, optional): Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks. Defaults to None.
        placement_constraints(List, optional): An array of placement constraint objects to use for tasks in your service. You can specify a maximum of 10 constraints per task (this limit includes constraints in the task definition and those specified at runtime). . Defaults to None.
        placement_strategy(List, optional): The placement strategy objects to use for tasks in your service. You can specify a maximum of five strategy rules per service. Defaults to None.
        network_configuration(Dict, optional): The network configuration for the service. This parameter is required for task definitions that use the awsvpc network mode to receive their own elastic network interface, and it is not supported for other network modes. For more information, see Task networking in the Amazon Elastic Container Service Developer Guide. Defaults to None.
        health_check_grace_period_seconds(int, optional): The period of time, in seconds, that the Amazon ECS service scheduler should ignore unhealthy Elastic Load Balancing target health checks after a task has first started. This is only used when your service is configured to use a load balancer. If your service has a load balancer defined and you don't specify a health check grace period value, the default value of 0 is used. If your service's tasks take a while to start and respond to Elastic Load Balancing health checks, you can specify a health check grace period of up to 2,147,483,647 seconds. During that time, the Amazon ECS service scheduler ignores health check status. This grace period can prevent the service scheduler from marking tasks as unhealthy and stopping them before they have time to come up. Defaults to None.
        scheduling_strategy(str, optional): The scheduling strategy to use for the service. For more information, see Services. There are two service scheduler strategies available:    REPLICA-The replica scheduling strategy places and maintains the desired number of tasks across your cluster. By default, the service scheduler spreads tasks across Availability Zones. You can use task placement strategies and constraints to customize task placement decisions. This scheduler strategy is required if the service is using the CODE_DEPLOY or EXTERNAL deployment controller types.    DAEMON-The daemon scheduling strategy deploys exactly one task on each active container instance that meets all of the task placement constraints that you specify in your cluster. The service scheduler also evaluates the task placement constraints for running tasks and will stop tasks that do not meet the placement constraints. When you're using this strategy, you don't need to specify a desired number of tasks, a task placement strategy, or use Service Auto Scaling policies.  Tasks using the Fargate launch type or the CODE_DEPLOY or EXTERNAL deployment controller types don't support the DAEMON scheduling strategy.   . Defaults to None.
        deployment_controller(Dict, optional): The deployment controller to use for the service. If no deployment controller is specified, the default value of ECS is used. Defaults to None.
        tags(List, optional): The metadata that you apply to the service to help you categorize and organize them. Each tag consists of a key and an optional value, both of which you define. When a service is deleted, the tags are deleted as well. The following basic restrictions apply to tags:   Maximum number of tags per resource - 50   For each resource, each tag key must be unique, and each tag key can have only one value.   Maximum key length - 128 Unicode characters in UTF-8   Maximum value length - 256 Unicode characters in UTF-8   If your tagging schema is used across multiple services and resources, remember that other services may have restrictions on allowed characters. Generally allowed characters are: letters, numbers, and spaces representable in UTF-8, and the following characters: + - = . _ : / @.   Tag keys and values are case-sensitive.   Do not use aws:, AWS:, or any upper or lowercase combination of such as a prefix for either keys or values as it is reserved for Amazon Web Services use. You cannot edit or delete tag keys or values with this prefix. Tags with this prefix do not count against your tags per resource limit.  . Defaults to None.
        enable_ecs_managed_tags(bool, optional): Specifies whether to enable Amazon ECS managed tags for the tasks within the service. For more information, see Tagging Your Amazon ECS Resources in the Amazon Elastic Container Service Developer Guide. Defaults to None.
        propagate_tags(str, optional): Specifies whether to propagate the tags from the task definition or the service to the tasks in the service. If no value is specified, the tags are not propagated. Tags can only be propagated to the tasks within the service during service creation. To add tags to a task after service creation, use the TagResource API action. Defaults to None.
        enable_execute_command(bool, optional): Whether or not the execute command functionality is enabled for the service. If true, this enables execute command functionality on all containers in the service tasks. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ecs.service.init.create(ctx, service_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ecs.service.init.create service_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ecs",
        operation="create_service",
        op_kwargs={
            "cluster": cluster,
            "serviceName": service_name,
            "taskDefinition": task_definition,
            "loadBalancers": load_balancers,
            "serviceRegistries": service_registries,
            "desiredCount": desired_count,
            "clientToken": client_token,
            "launchType": launch_type,
            "capacityProviderStrategy": capacity_provider_strategy,
            "platformVersion": platform_version,
            "role": role,
            "deploymentConfiguration": deployment_configuration,
            "placementConstraints": placement_constraints,
            "placementStrategy": placement_strategy,
            "networkConfiguration": network_configuration,
            "healthCheckGracePeriodSeconds": health_check_grace_period_seconds,
            "schedulingStrategy": scheduling_strategy,
            "deploymentController": deployment_controller,
            "tags": tags,
            "enableECSManagedTags": enable_ecs_managed_tags,
            "propagateTags": propagate_tags,
            "enableExecuteCommand": enable_execute_command,
        },
    )


async def delete(
    hub, ctx, service: str, cluster: str = None, force: bool = None
) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a specified service within a cluster. You can delete a service if you have no running tasks in it and the desired task count is zero. If the service is actively maintaining tasks, you cannot delete it, and you must update the service to a desired task count of zero. For more information, see UpdateService.  When you delete a service, if there are still running tasks that require cleanup, the service status moves from ACTIVE to DRAINING, and the service is no longer visible in the console or in the ListServices API operation. After all tasks have transitioned to either STOPPING or STOPPED status, the service status moves from DRAINING to INACTIVE. Services in the DRAINING or INACTIVE status can still be viewed with the DescribeServices API operation. However, in the future, INACTIVE services may be cleaned up and purged from Amazon ECS record keeping, and DescribeServices calls on those services return a ServiceNotFoundException error.   If you attempt to create a new service with the same name as an existing service in either ACTIVE or DRAINING status, you receive an error.

    Args:
        cluster(str, optional): The short name or full Amazon Resource Name (ARN) of the cluster that hosts the service to delete. If you do not specify a cluster, the default cluster is assumed. Defaults to None.
        service(str): The name of the service to delete.
        force(bool, optional): If true, allows you to delete a service even if it has not been scaled down to zero tasks. It is only necessary to use this if the service is using the REPLICA scheduling strategy. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ecs.service.init.delete(ctx, service=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ecs.service.init.delete service=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ecs",
        operation="delete_service",
        op_kwargs={"cluster": cluster, "service": service, "force": force},
    )


async def describe_all(
    hub, ctx, services: List, cluster: str = None, include: List = None
) -> Dict:
    r"""
    **Autogenerated function**

    Describes the specified services running in your cluster.

    Args:
        cluster(str, optional): The short name or full Amazon Resource Name (ARN)the cluster that hosts the service to describe. If you do not specify a cluster, the default cluster is assumed. This parameter is required if the service or services you are describing were launched in any cluster other than the default cluster. Defaults to None.
        services(List): A list of services to describe. You may specify up to 10 services to describe in a single operation.
        include(List, optional): Specifies whether you want to see the resource tags for the service. If TAGS is specified, the tags are included in the response. If this field is omitted, tags are not included in the response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ecs.service.init.describe_all(ctx, services=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ecs.service.init.describe_all services=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ecs",
        operation="describe_services",
        op_kwargs={"cluster": cluster, "services": services, "include": include},
    )


async def list_all(
    hub,
    ctx,
    cluster: str = None,
    next_token: str = None,
    max_results: int = None,
    launch_type: str = None,
    scheduling_strategy: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of services. You can filter the results by cluster, launch type, and scheduling strategy.

    Args:
        cluster(str, optional): The short name or full Amazon Resource Name (ARN) of the cluster to use when filtering the ListServices results. If you do not specify a cluster, the default cluster is assumed. Defaults to None.
        next_token(str, optional): The nextToken value returned from a ListServices request indicating that more results are available to fulfill the request and further calls will be needed. If maxResults was provided, it is possible the number of results to be fewer than maxResults.  This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. . Defaults to None.
        max_results(int, optional): The maximum number of service results returned by ListServices in paginated output. When this parameter is used, ListServices only returns maxResults results in a single page along with a nextToken response element. The remaining results of the initial request can be seen by sending another ListServices request with the returned nextToken value. This value can be between 1 and 100. If this parameter is not used, then ListServices returns up to 10 results and a nextToken value if applicable. Defaults to None.
        launch_type(str, optional): The launch type to use when filtering the ListServices results. Defaults to None.
        scheduling_strategy(str, optional): The scheduling strategy to use when filtering the ListServices results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ecs.service.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ecs.service.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ecs",
        operation="list_services",
        op_kwargs={
            "cluster": cluster,
            "nextToken": next_token,
            "maxResults": max_results,
            "launchType": launch_type,
            "schedulingStrategy": scheduling_strategy,
        },
    )


async def update(
    hub,
    ctx,
    service: str,
    cluster: str = None,
    desired_count: int = None,
    task_definition: str = None,
    capacity_provider_strategy: List = None,
    deployment_configuration: Dict = None,
    network_configuration: Dict = None,
    placement_constraints: List = None,
    placement_strategy: List = None,
    platform_version: str = None,
    force_new_deployment: bool = None,
    health_check_grace_period_seconds: int = None,
    enable_execute_command: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Updating the task placement strategies and constraints on an Amazon ECS service remains in preview and is a Beta Service as defined by and subject to the Beta Service Participation Service Terms located at https://aws.amazon.com/service-terms ("Beta Terms"). These Beta Terms apply to your participation in this preview.  Modifies the parameters of a service. For services using the rolling update (ECS) deployment controller, the desired count, deployment configuration, network configuration, task placement constraints and strategies, or task definition used can be updated. For services using the blue/green (CODE_DEPLOY) deployment controller, only the desired count, deployment configuration, task placement constraints and strategies, and health check grace period can be updated using this API. If the network configuration, platform version, or task definition need to be updated, a new CodeDeploy deployment should be created. For more information, see CreateDeployment in the CodeDeploy API Reference. For services using an external deployment controller, you can update only the desired count, task placement constraints and strategies, and health check grace period using this API. If the launch type, load balancer, network configuration, platform version, or task definition need to be updated, you should create a new task set. For more information, see CreateTaskSet. You can add to or subtract from the number of instantiations of a task definition in a service by specifying the cluster that the service is running in and a new desiredCount parameter. If you have updated the Docker image of your application, you can create a new task definition with that image and deploy it to your service. The service scheduler uses the minimum healthy percent and maximum percent parameters (in the service's deployment configuration) to determine the deployment strategy.  If your updated Docker image uses the same tag as what is in the existing task definition for your service (for example, my_image:latest), you do not need to create a new revision of your task definition. You can update the service using the forceNewDeployment option. The new tasks launched by the deployment pull the current image/tag combination from your repository when they start.  You can also update the deployment configuration of a service. When a deployment is triggered by updating the task definition of a service, the service scheduler uses the deployment configuration parameters, minimumHealthyPercent and maximumPercent, to determine the deployment strategy.   If minimumHealthyPercent is below 100%, the scheduler can ignore desiredCount temporarily during a deployment. For example, if desiredCount is four tasks, a minimum of 50% allows the scheduler to stop two existing tasks before starting two new tasks. Tasks for services that do not use a load balancer are considered healthy if they are in the RUNNING state. Tasks for services that use a load balancer are considered healthy if they are in the RUNNING state and the container instance they are hosted on is reported as healthy by the load balancer.   The maximumPercent parameter represents an upper limit on the number of running tasks during a deployment, which enables you to define the deployment batch size. For example, if desiredCount is four tasks, a maximum of 200% starts four new tasks before stopping the four older tasks (provided that the cluster resources required to do this are available).   When UpdateService stops a task during a deployment, the equivalent of docker stop is issued to the containers running in the task. This results in a SIGTERM and a 30-second timeout, after which SIGKILL is sent and the containers are forcibly stopped. If the container handles the SIGTERM gracefully and exits within 30 seconds from receiving it, no SIGKILL is sent. When the service scheduler launches new tasks, it determines task placement in your cluster with the following logic:   Determine which of the container instances in your cluster can support your service's task definition (for example, they have the required CPU, memory, ports, and container instance attributes).   By default, the service scheduler attempts to balance tasks across Availability Zones in this manner (although you can choose a different placement strategy):   Sort the valid container instances by the fewest number of running tasks for this service in the same Availability Zone as the instance. For example, if zone A has one running service task and zones B and C each have zero, valid container instances in either zone B or C are considered optimal for placement.   Place the new service task on a valid container instance in an optimal Availability Zone (based on the previous steps), favoring container instances with the fewest number of running tasks for this service.     When the service scheduler stops running tasks, it attempts to maintain balance across the Availability Zones in your cluster using the following logic:    Sort the container instances by the largest number of running tasks for this service in the same Availability Zone as the instance. For example, if zone A has one running service task and zones B and C each have two, container instances in either zone B or C are considered optimal for termination.   Stop the task on a container instance in an optimal Availability Zone (based on the previous steps), favoring container instances with the largest number of running tasks for this service.

    Args:
        cluster(str, optional): The short name or full Amazon Resource Name (ARN) of the cluster that your service is running on. If you do not specify a cluster, the default cluster is assumed. Defaults to None.
        service(str): The name of the service to update.
        desired_count(int, optional): The number of instantiations of the task to place and keep running in your service. Defaults to None.
        task_definition(str, optional): The family and revision (family:revision) or full ARN of the task definition to run in your service. If a revision is not specified, the latest ACTIVE revision is used. If you modify the task definition with UpdateService, Amazon ECS spawns a task with the new version of the task definition and then stops an old task after the new version is running. Defaults to None.
        capacity_provider_strategy(List, optional): The capacity provider strategy to update the service to use. If the service is using the default capacity provider strategy for the cluster, the service can be updated to use one or more capacity providers as opposed to the default capacity provider strategy. However, when a service is using a capacity provider strategy that is not the default capacity provider strategy, the service cannot be updated to use the cluster's default capacity provider strategy. A capacity provider strategy consists of one or more capacity providers along with the base and weight to assign to them. A capacity provider must be associated with the cluster to be used in a capacity provider strategy. The PutClusterCapacityProviders API is used to associate a capacity provider with a cluster. Only capacity providers with an ACTIVE or UPDATING status can be used. If specifying a capacity provider that uses an Auto Scaling group, the capacity provider must already be created. New capacity providers can be created with the CreateCapacityProvider API operation. To use a Fargate capacity provider, specify either the FARGATE or FARGATE_SPOT capacity providers. The Fargate capacity providers are available to all accounts and only need to be associated with a cluster to be used. The PutClusterCapacityProviders API operation is used to update the list of available capacity providers for a cluster after the cluster is created. . Defaults to None.
        deployment_configuration(Dict, optional): Optional deployment parameters that control how many tasks run during the deployment and the ordering of stopping and starting tasks. Defaults to None.
        network_configuration(Dict, optional): An object representing the network configuration for the service. Defaults to None.
        placement_constraints(List, optional): An array of task placement constraint objects to update the service to use. If no value is specified, the existing placement constraints for the service will remain unchanged. If this value is specified, it will override any existing placement constraints defined for the service. To remove all existing placement constraints, specify an empty array. You can specify a maximum of 10 constraints per task (this limit includes constraints in the task definition and those specified at runtime). Defaults to None.
        placement_strategy(List, optional): The task placement strategy objects to update the service to use. If no value is specified, the existing placement strategy for the service will remain unchanged. If this value is specified, it will override the existing placement strategy defined for the service. To remove an existing placement strategy, specify an empty object. You can specify a maximum of five strategy rules per service. Defaults to None.
        platform_version(str, optional): The platform version on which your tasks in the service are running. A platform version is only specified for tasks using the Fargate launch type. If a platform version is not specified, the LATEST platform version is used by default. For more information, see Fargate Platform Versions in the Amazon Elastic Container Service Developer Guide. Defaults to None.
        force_new_deployment(bool, optional): Whether to force a new deployment of the service. Deployments are not forced by default. You can use this option to trigger a new deployment with no service definition changes. For example, you can update a service's tasks to use a newer Docker image with the same image/tag combination (my_image:latest) or to roll Fargate tasks onto a newer platform version. Defaults to None.
        health_check_grace_period_seconds(int, optional): The period of time, in seconds, that the Amazon ECS service scheduler should ignore unhealthy Elastic Load Balancing target health checks after a task has first started. This is only valid if your service is configured to use a load balancer. If your service's tasks take a while to start and respond to Elastic Load Balancing health checks, you can specify a health check grace period of up to 2,147,483,647 seconds. During that time, the Amazon ECS service scheduler ignores the Elastic Load Balancing health check status. This grace period can prevent the ECS service scheduler from marking tasks as unhealthy and stopping them before they have time to come up. Defaults to None.
        enable_execute_command(bool, optional): If true, this enables execute command functionality on all task containers. If you do not want to override the value that was set when the service was created, you can set this to null when performing this action. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ecs.service.init.update(ctx, service=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ecs.service.init.update service=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ecs",
        operation="update_service",
        op_kwargs={
            "cluster": cluster,
            "service": service,
            "desiredCount": desired_count,
            "taskDefinition": task_definition,
            "capacityProviderStrategy": capacity_provider_strategy,
            "deploymentConfiguration": deployment_configuration,
            "networkConfiguration": network_configuration,
            "placementConstraints": placement_constraints,
            "placementStrategy": placement_strategy,
            "platformVersion": platform_version,
            "forceNewDeployment": force_new_deployment,
            "healthCheckGracePeriodSeconds": health_check_grace_period_seconds,
            "enableExecuteCommand": enable_execute_command,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe_all(
    hub, ctx, authentication_token: str, limit: int = None, marker: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Describes the current user's special folders; the RootFolder and the RecycleBin. RootFolder is the root of user's files and folders and RecycleBin is the root of recycled items. This is not a valid action for SigV4 (administrative API) clients. This action requires an authentication token. To get an authentication token, register an application with Amazon WorkDocs. For more information, see Authentication and Access Control for User Applications in the Amazon WorkDocs Developer Guide.

    Args:
        authentication_token(str): Amazon WorkDocs authentication token.
        limit(int, optional): The maximum number of items to return. Defaults to None.
        marker(str, optional): The marker for the next set of results. (You received this marker from a previous call.). Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.workdoc.root_folder.describe_all(
                ctx, authentication_token=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.workdoc.root_folder.describe_all authentication_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="workdocs",
        operation="describe_root_folders",
        op_kwargs={
            "AuthenticationToken": authentication_token,
            "Limit": limit,
            "Marker": marker,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def restore(
    hub,
    ctx,
    db_cluster_identifier: str,
    snapshot_identifier: str,
    engine: str,
    availability_zones: List = None,
    engine_version: str = None,
    port: int = None,
    db_subnet_group_name: str = None,
    vpc_security_group_ids: List = None,
    tags: List = None,
    kms_key_id: str = None,
    enable_cloudwatch_logs_exports: List = None,
    deletion_protection: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a new cluster from a snapshot or cluster snapshot. If a snapshot is specified, the target cluster is created from the source DB snapshot with a default configuration and default security group. If a cluster snapshot is specified, the target cluster is created from the source cluster restore point with the same configuration as the original source DB cluster, except that the new cluster is created with the default security group.

    Args:
        availability_zones(List, optional): Provides the list of Amazon EC2 Availability Zones that instances in the restored DB cluster can be created in. Defaults to None.
        db_cluster_identifier(str): The name of the cluster to create from the snapshot or cluster snapshot. This parameter isn't case sensitive. Constraints:   Must contain from 1 to 63 letters, numbers, or hyphens.   The first character must be a letter.   Cannot end with a hyphen or contain two consecutive hyphens.   Example: my-snapshot-id .
        snapshot_identifier(str): The identifier for the snapshot or cluster snapshot to restore from. You can use either the name or the Amazon Resource Name (ARN) to specify a cluster snapshot. However, you can use only the ARN to specify a snapshot. Constraints:   Must match the identifier of an existing snapshot.  .
        engine(str): The database engine to use for the new cluster. Default: The same as source. Constraint: Must be compatible with the engine of the source.
        engine_version(str, optional): The version of the database engine to use for the new cluster. Defaults to None.
        port(int, optional): The port number on which the new cluster accepts connections. Constraints: Must be a value from 1150 to 65535. Default: The same port as the original cluster. Defaults to None.
        db_subnet_group_name(str, optional): The name of the subnet group to use for the new cluster. Constraints: If provided, must match the name of an existing DBSubnetGroup. Example: mySubnetgroup . Defaults to None.
        vpc_security_group_ids(List, optional): A list of virtual private cloud (VPC) security groups that the new cluster will belong to. Defaults to None.
        tags(List, optional): The tags to be assigned to the restored cluster. Defaults to None.
        kms_key_id(str, optional): The KMS key identifier to use when restoring an encrypted cluster from a DB snapshot or cluster snapshot. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are restoring a cluster with the same account that owns the KMS encryption key used to encrypt the new cluster, then you can use the KMS key alias instead of the ARN for the KMS encryption key. If you do not specify a value for the KmsKeyId parameter, then the following occurs:   If the snapshot or cluster snapshot in SnapshotIdentifier is encrypted, then the restored cluster is encrypted using the KMS key that was used to encrypt the snapshot or the cluster snapshot.   If the snapshot or the cluster snapshot in SnapshotIdentifier is not encrypted, then the restored DB cluster is not encrypted.  . Defaults to None.
        enable_cloudwatch_logs_exports(List, optional): A list of log types that must be enabled for exporting to Amazon CloudWatch Logs. Defaults to None.
        deletion_protection(bool, optional): Specifies whether this cluster can be deleted. If DeletionProtection is enabled, the cluster cannot be deleted unless it is modified and DeletionProtection is disabled. DeletionProtection protects clusters from being accidentally deleted. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.docdb.db.cluster.from_snapshot.restore(
                ctx, db_cluster_identifier=value, snapshot_identifier=value, engine=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.docdb.db.cluster.from_snapshot.restore db_cluster_identifier=value, snapshot_identifier=value, engine=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="docdb",
        operation="restore_db_cluster_from_snapshot",
        op_kwargs={
            "AvailabilityZones": availability_zones,
            "DBClusterIdentifier": db_cluster_identifier,
            "SnapshotIdentifier": snapshot_identifier,
            "Engine": engine,
            "EngineVersion": engine_version,
            "Port": port,
            "DBSubnetGroupName": db_subnet_group_name,
            "VpcSecurityGroupIds": vpc_security_group_ids,
            "Tags": tags,
            "KmsKeyId": kms_key_id,
            "EnableCloudwatchLogsExports": enable_cloudwatch_logs_exports,
            "DeletionProtection": deletion_protection,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def copy(
    hub,
    ctx,
    domain: str,
    source_repository: str,
    destination_repository: str,
    format_: str,
    package: str,
    domain_owner: str = None,
    namespace: str = None,
    versions: List = None,
    version_revisions: Dict = None,
    allow_overwrite: bool = None,
    include_from_upstream: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Copies package versions from one repository to another repository in the same domain.    You must specify versions or versionRevisions. You cannot specify both.

    Args:
        domain(str):  The name of the domain that contains the source and destination repositories. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        source_repository(str):  The name of the repository that contains the package versions to copy. .
        destination_repository(str):  The name of the repository into which package versions are copied. .
        format_(str):  The format of the package that is copied. The valid package types are:     npm: A Node Package Manager (npm) package.     pypi: A Python Package Index (PyPI) package.     maven: A Maven package that contains compiled code in a distributable format, such as a JAR file.   .
        namespace(str, optional):  The namespace of the package. The package component that specifies its namespace depends on its type. For example:     The namespace of a Maven package is its groupId.     The namespace of an npm package is its scope.     A Python package does not contain a corresponding component, so Python packages do not have a namespace.   . Defaults to None.
        package(str):  The name of the package that is copied. .
        versions(List, optional):  The versions of the package to copy.    You must specify versions or versionRevisions. You cannot specify both.  . Defaults to None.
        version_revisions(Dict, optional):  A list of key-value pairs. The keys are package versions and the values are package version revisions. A CopyPackageVersion operation succeeds if the specified versions in the source repository match the specified package version revision.    You must specify versions or versionRevisions. You cannot specify both.  . Defaults to None.
        allow_overwrite(bool, optional):  Set to true to overwrite a package version that already exists in the destination repository. If set to false and the package version already exists in the destination repository, the package version is returned in the failedVersions field of the response with an ALREADY_EXISTS error code. . Defaults to None.
        include_from_upstream(bool, optional):  Set to true to copy packages from repositories that are upstream from the source repository to the destination repository. The default setting is false. For more information, see Working with upstream repositories. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.package.version.init.copy(
                ctx,
                domain=value,
                source_repository=value,
                destination_repository=value,
                format_=value,
                package=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.package.version.init.copy domain=value, source_repository=value, destination_repository=value, format_=value, package=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="copy_package_versions",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "sourceRepository": source_repository,
            "destinationRepository": destination_repository,
            "format": format_,
            "namespace": namespace,
            "package": package,
            "versions": versions,
            "versionRevisions": version_revisions,
            "allowOverwrite": allow_overwrite,
            "includeFromUpstream": include_from_upstream,
        },
    )


async def delete_multiple(
    hub,
    ctx,
    domain: str,
    repository: str,
    format_: str,
    package: str,
    versions: List,
    domain_owner: str = None,
    namespace: str = None,
    expected_status: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Deletes one or more versions of a package. A deleted package version cannot be restored in your repository. If you want to remove a package version from your repository and be able to restore it later, set its status to Archived. Archived packages cannot be downloaded from a repository and don't show up with list package APIs (for example, ListackageVersions), but you can restore them using UpdatePackageVersionsStatus.

    Args:
        domain(str):  The name of the domain that contains the package to delete. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        repository(str):  The name of the repository that contains the package versions to delete. .
        format_(str):  The format of the package versions to delete. The valid values are:     npm     pypi     maven   .
        namespace(str, optional):  The namespace of the package. The package component that specifies its namespace depends on its type. For example:     The namespace of a Maven package is its groupId.     The namespace of an npm package is its scope.     A Python package does not contain a corresponding component, so Python packages do not have a namespace.   . Defaults to None.
        package(str):  The name of the package with the versions to delete. .
        versions(List):  An array of strings that specify the versions of the package to delete. .
        expected_status(str, optional):  The expected status of the package version to delete. Valid values are:     Published     Unfinished     Unlisted     Archived     Disposed   . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.package.version.init.delete_multiple(
                ctx, domain=value, repository=value, format_=value, package=value, versions=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.package.version.init.delete_multiple domain=value, repository=value, format_=value, package=value, versions=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="delete_package_versions",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "repository": repository,
            "format": format_,
            "namespace": namespace,
            "package": package,
            "versions": versions,
            "expectedStatus": expected_status,
        },
    )


async def describe(
    hub,
    ctx,
    domain: str,
    repository: str,
    format_: str,
    package: str,
    package_version: str,
    domain_owner: str = None,
    namespace: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Returns a PackageVersionDescription object that contains information about the requested package version.

    Args:
        domain(str):  The name of the domain that contains the repository that contains the package version. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        repository(str):  The name of the repository that contains the package version. .
        format_(str):  A format that specifies the type of the requested package version. The valid values are:     npm     pypi     maven   .
        namespace(str, optional):  The namespace of the package. The package component that specifies its namespace depends on its type. For example:     The namespace of a Maven package is its groupId.     The namespace of an npm package is its scope.     A Python package does not contain a corresponding component, so Python packages do not have a namespace.   . Defaults to None.
        package(str):  The name of the requested package version. .
        package_version(str):  A string that contains the package version (for example, 3.5.2). .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.package.version.init.describe(
                ctx,
                domain=value,
                repository=value,
                format_=value,
                package=value,
                package_version=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.package.version.init.describe domain=value, repository=value, format_=value, package=value, package_version=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="describe_package_version",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "repository": repository,
            "format": format_,
            "namespace": namespace,
            "package": package,
            "packageVersion": package_version,
        },
    )


async def dispose(
    hub,
    ctx,
    domain: str,
    repository: str,
    format_: str,
    package: str,
    versions: List,
    domain_owner: str = None,
    namespace: str = None,
    version_revisions: Dict = None,
    expected_status: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Deletes the assets in package versions and sets the package versions' status to Disposed. A disposed package version cannot be restored in your repository because its assets are deleted.   To view all disposed package versions in a repository, use ListPackageVersions and set the status parameter to Disposed.   To view information about a disposed package version, use DescribePackageVersion.

    Args:
        domain(str):  The name of the domain that contains the repository you want to dispose. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        repository(str):  The name of the repository that contains the package versions you want to dispose. .
        format_(str):  A format that specifies the type of package versions you want to dispose. The valid values are:     npm     pypi     maven   .
        namespace(str, optional):  The namespace of the package. The package component that specifies its namespace depends on its type. For example:     The namespace of a Maven package is its groupId.     The namespace of an npm package is its scope.     A Python package does not contain a corresponding component, so Python packages do not have a namespace.   . Defaults to None.
        package(str):  The name of the package with the versions you want to dispose. .
        versions(List):  The versions of the package you want to dispose. .
        version_revisions(Dict, optional):  The revisions of the package versions you want to dispose. . Defaults to None.
        expected_status(str, optional):  The expected status of the package version to dispose. Valid values are:     Published     Unfinished     Unlisted     Archived     Disposed   . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.package.version.init.dispose(
                ctx, domain=value, repository=value, format_=value, package=value, versions=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.package.version.init.dispose domain=value, repository=value, format_=value, package=value, versions=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="dispose_package_versions",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "repository": repository,
            "format": format_,
            "namespace": namespace,
            "package": package,
            "versions": versions,
            "versionRevisions": version_revisions,
            "expectedStatus": expected_status,
        },
    )


async def list_all(
    hub,
    ctx,
    domain: str,
    repository: str,
    format_: str,
    package: str,
    domain_owner: str = None,
    namespace: str = None,
    status: str = None,
    sort_by: str = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Returns a list of PackageVersionSummary objects for package versions in a repository that match the request parameters.

    Args:
        domain(str):  The name of the domain that contains the repository that contains the returned package versions. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        repository(str):  The name of the repository that contains the package. .
        format_(str):  The format of the returned packages. The valid package types are:     npm: A Node Package Manager (npm) package.     pypi: A Python Package Index (PyPI) package.     maven: A Maven package that contains compiled code in a distributable format, such as a JAR file.   .
        namespace(str, optional):  The namespace of the package. The package component that specifies its namespace depends on its type. For example:     The namespace of a Maven package is its groupId.     The namespace of an npm package is its scope.     A Python package does not contain a corresponding component, so Python packages do not have a namespace.   . Defaults to None.
        package(str):  The name of the package for which you want to return a list of package versions. .
        status(str, optional):  A string that specifies the status of the package versions to include in the returned list. It can be one of the following:     Published     Unfinished     Unlisted     Archived     Disposed   . Defaults to None.
        sort_by(str, optional):  How to sort the returned list of package versions. . Defaults to None.
        max_results(int, optional):  The maximum number of results to return per page. . Defaults to None.
        next_token(str, optional):  The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.package.version.init.list_all(
                ctx, domain=value, repository=value, format_=value, package=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.package.version.init.list_all domain=value, repository=value, format_=value, package=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="list_package_versions",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "repository": repository,
            "format": format_,
            "namespace": namespace,
            "package": package,
            "status": status,
            "sortBy": sort_by,
            "maxResults": max_results,
            "nextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, account_id: str, name: str) -> None:
    r"""
    **Autogenerated function**

    Deletes the access point policy for the specified access point.  All Amazon S3 on Outposts REST API requests for this action require an additional parameter of x-amz-outpost-id to be passed with the request and an S3 on Outposts endpoint hostname prefix instead of s3-control. For an example of the request syntax for Amazon S3 on Outposts that uses the S3 on Outposts endpoint hostname prefix and the x-amz-outpost-id derived using the access point ARN, see the Examples section. The following actions are related to DeleteAccessPointPolicy:    PutAccessPointPolicy     GetAccessPointPolicy

    Args:
        account_id(str): The account ID for the account that owns the specified access point.
        name(str): The name of the access point whose policy you want to delete. For using this parameter with Amazon S3 on Outposts with the REST API, you must specify the name and the x-amz-outpost-id as well. For using this parameter with S3 on Outposts with the Amazon Web Services SDK and CLI, you must specify the ARN of the access point accessed in the format arn:aws:s3-outposts:<Region>:<account-id>:outpost/<outpost-id>/accesspoint/<my-accesspoint-name>. For example, to access the access point reports-ap through outpost my-outpost owned by account 123456789012 in Region us-west-2, use the URL encoding of arn:aws:s3-outposts:us-west-2:123456789012:outpost/my-outpost/accesspoint/reports-ap. The value must be URL encoded. .
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3control.access_point.policy.init.delete(
                ctx, account_id=value, name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3control.access_point.policy.init.delete account_id=value, name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3control",
        operation="delete_access_point_policy",
        op_kwargs={"AccountId": account_id, "Name": name},
    )


async def get(hub, ctx, account_id: str, name: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the access point policy associated with the specified access point. The following actions are related to GetAccessPointPolicy:    PutAccessPointPolicy     DeleteAccessPointPolicy

    Args:
        account_id(str): The account ID for the account that owns the specified access point.
        name(str): The name of the access point whose policy you want to retrieve. For using this parameter with Amazon S3 on Outposts with the REST API, you must specify the name and the x-amz-outpost-id as well. For using this parameter with S3 on Outposts with the Amazon Web Services SDK and CLI, you must specify the ARN of the access point accessed in the format arn:aws:s3-outposts:<Region>:<account-id>:outpost/<outpost-id>/accesspoint/<my-accesspoint-name>. For example, to access the access point reports-ap through outpost my-outpost owned by account 123456789012 in Region us-west-2, use the URL encoding of arn:aws:s3-outposts:us-west-2:123456789012:outpost/my-outpost/accesspoint/reports-ap. The value must be URL encoded. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3control.access_point.policy.init.get(
                ctx, account_id=value, name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3control.access_point.policy.init.get account_id=value, name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3control",
        operation="get_access_point_policy",
        op_kwargs={"AccountId": account_id, "Name": name},
    )


async def put(hub, ctx, account_id: str, name: str, policy: str) -> None:
    r"""
    **Autogenerated function**

    Associates an access policy with the specified access point. Each access point can have only one policy, so a request made to this API replaces any existing policy associated with the specified access point.  All Amazon S3 on Outposts REST API requests for this action require an additional parameter of x-amz-outpost-id to be passed with the request and an S3 on Outposts endpoint hostname prefix instead of s3-control. For an example of the request syntax for Amazon S3 on Outposts that uses the S3 on Outposts endpoint hostname prefix and the x-amz-outpost-id derived using the access point ARN, see the Examples section. The following actions are related to PutAccessPointPolicy:    GetAccessPointPolicy     DeleteAccessPointPolicy

    Args:
        account_id(str): The account ID for owner of the bucket associated with the specified access point.
        name(str): The name of the access point that you want to associate with the specified policy. For using this parameter with Amazon S3 on Outposts with the REST API, you must specify the name and the x-amz-outpost-id as well. For using this parameter with S3 on Outposts with the Amazon Web Services SDK and CLI, you must specify the ARN of the access point accessed in the format arn:aws:s3-outposts:<Region>:<account-id>:outpost/<outpost-id>/accesspoint/<my-accesspoint-name>. For example, to access the access point reports-ap through outpost my-outpost owned by account 123456789012 in Region us-west-2, use the URL encoding of arn:aws:s3-outposts:us-west-2:123456789012:outpost/my-outpost/accesspoint/reports-ap. The value must be URL encoded. .
        policy(str): The policy that you want to apply to the specified access point. For more information about access point policies, see Managing data access with Amazon S3 access points in the Amazon S3 User Guide.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3control.access_point.policy.init.put(
                ctx, account_id=value, name=value, policy=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3control.access_point.policy.init.put account_id=value, name=value, policy=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3control",
        operation="put_access_point_policy",
        op_kwargs={"AccountId": account_id, "Name": name, "Policy": policy},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    subscription_name: str,
    sns_topic_arn: str,
    source_type: str = None,
    event_categories: List = None,
    source_ids: List = None,
    enabled: bool = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Creates an DMS event notification subscription.  You can specify the type of source (SourceType) you want to be notified of, provide a list of DMS source IDs (SourceIds) that triggers the events, and provide a list of event categories (EventCategories) for events you want to be notified of. If you specify both the SourceType and SourceIds, such as SourceType = replication-instance and SourceIdentifier = my-replinstance, you will be notified of all the replication instance events for the specified source. If you specify a SourceType but don't specify a SourceIdentifier, you receive notice of the events for that source type for all your DMS sources. If you don't specify either SourceType nor SourceIdentifier, you will be notified of events generated from all DMS sources belonging to your customer account. For more information about DMS events, see Working with Events and Notifications in the Database Migration Service User Guide.

    Args:
        subscription_name(str): The name of the DMS event notification subscription. This name must be less than 255 characters.
        sns_topic_arn(str):  The Amazon Resource Name (ARN) of the Amazon SNS topic created for event notification. The ARN is created by Amazon SNS when you create a topic and subscribe to it. .
        source_type(str, optional):  The type of DMS resource that generates the events. For example, if you want to be notified of events generated by a replication instance, you set this parameter to replication-instance. If this value isn't specified, all events are returned.  Valid values: replication-instance | replication-task . Defaults to None.
        event_categories(List, optional): A list of event categories for a source type that you want to subscribe to. For more information, see Working with Events and Notifications in the Database Migration Service User Guide. . Defaults to None.
        source_ids(List, optional): A list of identifiers for which DMS provides notification events. If you don't specify a value, notifications are provided for all sources. If you specify multiple values, they must be of the same type. For example, if you specify a database instance ID, then all of the other values must be database instance IDs. Defaults to None.
        enabled(bool, optional):  A Boolean value; set to true to activate the subscription, or set to false to create the subscription but not activate it. . Defaults to None.
        tags(List, optional): One or more tags to be assigned to the event subscription. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.event.subscription.create(
                ctx, subscription_name=value, sns_topic_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.event.subscription.create subscription_name=value, sns_topic_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="create_event_subscription",
        op_kwargs={
            "SubscriptionName": subscription_name,
            "SnsTopicArn": sns_topic_arn,
            "SourceType": source_type,
            "EventCategories": event_categories,
            "SourceIds": source_ids,
            "Enabled": enabled,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, subscription_name: str) -> Dict:
    r"""
    **Autogenerated function**

     Deletes an DMS event subscription.

    Args:
        subscription_name(str): The name of the DMS event notification subscription to be deleted.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.event.subscription.delete(ctx, subscription_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.event.subscription.delete subscription_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="delete_event_subscription",
        op_kwargs={"SubscriptionName": subscription_name},
    )


async def describe_all(
    hub,
    ctx,
    subscription_name: str = None,
    filters: List = None,
    max_records: int = None,
    marker: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists all the event subscriptions for a customer account. The description of a subscription includes SubscriptionName, SNSTopicARN, CustomerID, SourceType, SourceID, CreationTime, and Status.  If you specify SubscriptionName, this action lists the description for that subscription.

    Args:
        subscription_name(str, optional): The name of the DMS event subscription to be described. Defaults to None.
        filters(List, optional): Filters applied to event subscriptions. Defaults to None.
        max_records(int, optional):  The maximum number of records to include in the response. If more records exist than the specified MaxRecords value, a pagination token called a marker is included in the response so that the remaining results can be retrieved.  Default: 100 Constraints: Minimum 20, maximum 100. Defaults to None.
        marker(str, optional):  An optional pagination token provided by a previous request. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by MaxRecords. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.event.subscription.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.event.subscription.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="describe_event_subscriptions",
        op_kwargs={
            "SubscriptionName": subscription_name,
            "Filters": filters,
            "MaxRecords": max_records,
            "Marker": marker,
        },
    )


async def modify(
    hub,
    ctx,
    subscription_name: str,
    sns_topic_arn: str = None,
    source_type: str = None,
    event_categories: List = None,
    enabled: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies an existing DMS event notification subscription.

    Args:
        subscription_name(str): The name of the DMS event notification subscription to be modified.
        sns_topic_arn(str, optional):  The Amazon Resource Name (ARN) of the Amazon SNS topic created for event notification. The ARN is created by Amazon SNS when you create a topic and subscribe to it. Defaults to None.
        source_type(str, optional):  The type of DMS resource that generates the events you want to subscribe to.  Valid values: replication-instance | replication-task. Defaults to None.
        event_categories(List, optional):  A list of event categories for a source type that you want to subscribe to. Use the DescribeEventCategories action to see a list of event categories. . Defaults to None.
        enabled(bool, optional):  A Boolean value; set to true to activate the subscription. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dms.event.subscription.modify(ctx, subscription_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dms.event.subscription.modify subscription_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dms",
        operation="modify_event_subscription",
        op_kwargs={
            "SubscriptionName": subscription_name,
            "SnsTopicArn": sns_topic_arn,
            "SourceType": source_type,
            "EventCategories": event_categories,
            "Enabled": enabled,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def redact(
    hub, ctx, account_id: str, conversation_id: str, message_id: str
) -> Dict:
    r"""
    **Autogenerated function**

    Redacts the specified message from the specified Amazon Chime conversation.

    Args:
        account_id(str): The Amazon Chime account ID.
        conversation_id(str): The conversation ID.
        message_id(str): The message ID.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.conversation_message.redact(
                ctx, account_id=value, conversation_id=value, message_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.conversation_message.redact account_id=value, conversation_id=value, message_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="redact_conversation_message",
        op_kwargs={
            "AccountId": account_id,
            "ConversationId": conversation_id,
            "MessageId": message_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    name: str,
    container: str,
    description: str = None,
    video: Dict = None,
    audio: Dict = None,
    thumbnails: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    The CreatePreset operation creates a preset with settings that you specify.  Elastic Transcoder checks the CreatePreset settings to ensure that they meet Elastic Transcoder requirements and to determine whether they comply with H.264 standards. If your settings are not valid for Elastic Transcoder, Elastic Transcoder returns an HTTP 400 response (ValidationException) and does not create the preset. If the settings are valid for Elastic Transcoder but aren't strictly compliant with the H.264 standard, Elastic Transcoder creates the preset and returns a warning message in the response. This helps you determine whether your settings comply with the H.264 standard while giving you greater flexibility with respect to the video that Elastic Transcoder produces.  Elastic Transcoder uses the H.264 video-compression format. For more information, see the International Telecommunication Union publication Recommendation ITU-T H.264: Advanced video coding for generic audiovisual services.

    Args:
        name(str): The name of the preset. We recommend that the name be unique within the AWS account, but uniqueness is not enforced.
        description(str, optional): A description of the preset. Defaults to None.
        container(str): The container type for the output file. Valid values include flac, flv, fmp4, gif, mp3, mp4, mpg, mxf, oga, ogg, ts, and webm.
        video(Dict, optional): A section of the request body that specifies the video parameters. Defaults to None.
        audio(Dict, optional): A section of the request body that specifies the audio parameters. Defaults to None.
        thumbnails(Dict, optional): A section of the request body that specifies the thumbnail parameters, if any. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elastictranscoder.preset.create(
                ctx, name=value, container=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elastictranscoder.preset.create name=value, container=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elastictranscoder",
        operation="create_preset",
        op_kwargs={
            "Name": name,
            "Description": description,
            "Container": container,
            "Video": video,
            "Audio": audio,
            "Thumbnails": thumbnails,
        },
    )


async def delete(hub, ctx, id_: str) -> Dict:
    r"""
    **Autogenerated function**

    The DeletePreset operation removes a preset that you've added in an AWS region.  You can't delete the default presets that are included with Elastic Transcoder.

    Args:
        id_(str): The identifier of the preset for which you want to get detailed information.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elastictranscoder.preset.delete(ctx, id_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elastictranscoder.preset.delete id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elastictranscoder",
        operation="delete_preset",
        op_kwargs={"Id": id_},
    )


async def list_all(hub, ctx, ascending: str = None, page_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    The ListPresets operation gets a list of the default presets included with Elastic Transcoder and the presets that you've added in an AWS region.

    Args:
        ascending(str, optional): To list presets in chronological order by the date and time that they were created, enter true. To list presets in reverse chronological order, enter false. Defaults to None.
        page_token(str, optional): When Elastic Transcoder returns more than one page of results, use pageToken in subsequent GET requests to get each successive page of results. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elastictranscoder.preset.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elastictranscoder.preset.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elastictranscoder",
        operation="list_presets",
        op_kwargs={"Ascending": ascending, "PageToken": page_token},
    )


async def read(hub, ctx, id_: str) -> Dict:
    r"""
    **Autogenerated function**

    The ReadPreset operation gets detailed information about a preset.

    Args:
        id_(str): The identifier of the preset for which you want to get detailed information.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elastictranscoder.preset.read(ctx, id_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elastictranscoder.preset.read id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elastictranscoder",
        operation="read_preset",
        op_kwargs={"Id": id_},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def update(hub, ctx, organization_id: str, entity_id: str, email: str) -> Dict:
    r"""
    **Autogenerated function**

    Updates the primary email for a user, group, or resource. The current email is moved into the list of aliases (or swapped between an existing alias and the current primary email), and the email provided in the input is promoted as the primary.

    Args:
        organization_id(str): The organization that contains the user, group, or resource to update.
        entity_id(str): The user, group, or resource to update.
        email(str): The value of the email to be updated as primary.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.workmail.primary_email_address.update(
                ctx, organization_id=value, entity_id=value, email=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.workmail.primary_email_address.update organization_id=value, entity_id=value, email=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="workmail",
        operation="update_primary_email_address",
        op_kwargs={
            "OrganizationId": organization_id,
            "EntityId": entity_id,
            "Email": email,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    client_request_token: str,
    external_meeting_id: str = None,
    meeting_host_id: str = None,
    media_region: str = None,
    tags: List = None,
    notifications_configuration: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Creates a new Amazon Chime SDK meeting in the specified media Region with no initial attendees. For more information about specifying media Regions, see Amazon Chime SDK Media Regions in the Amazon Chime Developer Guide . For more information about the Amazon Chime SDK, see Using the Amazon Chime SDK in the Amazon Chime Developer Guide .

    Args:
        client_request_token(str): The unique identifier for the client request. Use a different token for different meetings.
        external_meeting_id(str, optional): The external meeting ID. Defaults to None.
        meeting_host_id(str, optional): Reserved. Defaults to None.
        media_region(str, optional):  The Region in which to create the meeting. Default: us-east-1.   Available values: af-south-1 , ap-northeast-1 , ap-northeast-2 , ap-south-1 , ap-southeast-1 , ap-southeast-2 , ca-central-1 , eu-central-1 , eu-north-1 , eu-south-1 , eu-west-1 , eu-west-2 , eu-west-3 , sa-east-1 , us-east-1 , us-east-2 , us-west-1 , us-west-2 . . Defaults to None.
        tags(List, optional): The tag key-value pairs. Defaults to None.
        notifications_configuration(Dict, optional): The configuration for resource targets to receive notifications when meeting and attendee events occur. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.meeting.init.create(ctx, client_request_token=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.meeting.init.create client_request_token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="create_meeting",
        op_kwargs={
            "ClientRequestToken": client_request_token,
            "ExternalMeetingId": external_meeting_id,
            "MeetingHostId": meeting_host_id,
            "MediaRegion": media_region,
            "Tags": tags,
            "NotificationsConfiguration": notifications_configuration,
        },
    )


async def delete(hub, ctx, meeting_id: str) -> None:
    r"""
    **Autogenerated function**

    Deletes the specified Amazon Chime SDK meeting. The operation deletes all attendees, disconnects all clients, and prevents new clients from joining the meeting. For more information about the Amazon Chime SDK, see Using the Amazon Chime SDK in the Amazon Chime Developer Guide.

    Args:
        meeting_id(str): The Amazon Chime SDK meeting ID.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.meeting.init.delete(ctx, meeting_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.meeting.init.delete meeting_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="delete_meeting",
        op_kwargs={"MeetingId": meeting_id},
    )


async def get(hub, ctx, meeting_id: str) -> Dict:
    r"""
    **Autogenerated function**

     Gets the Amazon Chime SDK meeting details for the specified meeting ID. For more information about the Amazon Chime SDK, see Using the Amazon Chime SDK in the Amazon Chime Developer Guide .

    Args:
        meeting_id(str): The Amazon Chime SDK meeting ID.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.meeting.init.get(ctx, meeting_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.meeting.init.get meeting_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="get_meeting",
        op_kwargs={"MeetingId": meeting_id},
    )


async def list_all(hub, ctx, next_token: str = None, max_results: int = None) -> Dict:
    r"""
    **Autogenerated function**

     Lists up to 100 active Amazon Chime SDK meetings. For more information about the Amazon Chime SDK, see Using the Amazon Chime SDK in the Amazon Chime Developer Guide.

    Args:
        next_token(str, optional): The token to use to retrieve the next page of results. Defaults to None.
        max_results(int, optional): The maximum number of results to return in a single call. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.meeting.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.meeting.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="list_meetings",
        op_kwargs={"NextToken": next_token, "MaxResults": max_results},
    )


async def tag(hub, ctx, meeting_id: str, tags: List) -> None:
    r"""
    **Autogenerated function**

    Applies the specified tags to the specified Amazon Chime SDK meeting.

    Args:
        meeting_id(str): The Amazon Chime SDK meeting ID.
        tags(List): The tag key-value pairs.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.meeting.init.tag(ctx, meeting_id=value, tags=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.meeting.init.tag meeting_id=value, tags=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="tag_meeting",
        op_kwargs={"MeetingId": meeting_id, "Tags": tags},
    )


async def untag(hub, ctx, meeting_id: str, tag_keys: List) -> None:
    r"""
    **Autogenerated function**

    Untags the specified tags from the specified Amazon Chime SDK meeting.

    Args:
        meeting_id(str): The Amazon Chime SDK meeting ID.
        tag_keys(List): The tag keys.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.meeting.init.untag(ctx, meeting_id=value, tag_keys=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.meeting.init.untag meeting_id=value, tag_keys=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="untag_meeting",
        op_kwargs={"MeetingId": meeting_id, "TagKeys": tag_keys},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, bucket: str, expected_bucket_owner: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Gets the Object Lock configuration for a bucket. The rule specified in the Object Lock configuration will be applied by default to every new object placed in the specified bucket. For more information, see Locking Objects.

    Args:
        bucket(str): The bucket whose Object Lock configuration you want to retrieve. When using this action with an access point, you must direct requests to the access point hostname. The access point hostname takes the form AccessPointName-AccountId.s3-accesspoint.Region.amazonaws.com. When using this action with an access point through the AWS SDKs, you provide the access point ARN in place of the bucket name. For more information about access point ARNs, see Using access points in the Amazon S3 User Guide.
        expected_bucket_owner(str, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3.object_.lock_configuration.get(ctx, bucket=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3.object_.lock_configuration.get bucket=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3",
        operation="get_object_lock_configuration",
        op_kwargs={"Bucket": bucket, "ExpectedBucketOwner": expected_bucket_owner},
    )


async def put(
    hub,
    ctx,
    bucket: str,
    object_lock_configuration: Dict = None,
    request_payer: str = None,
    token: str = None,
    content_md5: str = None,
    expected_bucket_owner: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Places an Object Lock configuration on the specified bucket. The rule specified in the Object Lock configuration will be applied by default to every new object placed in the specified bucket. For more information, see Locking Objects.     The DefaultRetention settings require both a mode and a period.   The DefaultRetention period can be either Days or Years but you must select one. You cannot specify Days and Years at the same time.   You can only enable Object Lock for new buckets. If you want to turn on Object Lock for an existing bucket, contact AWS Support.

    Args:
        bucket(str): The bucket whose Object Lock configuration you want to create or replace.
        object_lock_configuration(Dict, optional): The Object Lock configuration that you want to apply to the specified bucket. Defaults to None.
        request_payer(str, optional): Confirms that the requester knows that they will be charged for the request. Bucket owners need not specify this parameter in their requests. For information about downloading objects from requester pays buckets, see Downloading Objects in Requestor Pays Buckets in the Amazon S3 User Guide. Defaults to None.
        token(str, optional): A token to allow Object Lock to be enabled for an existing bucket. Defaults to None.
        content_md5(str, optional): The MD5 hash for the request body. For requests made using the AWS Command Line Interface (CLI) or AWS SDKs, this field is calculated automatically. Defaults to None.
        expected_bucket_owner(str, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3.object_.lock_configuration.put(ctx, bucket=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3.object_.lock_configuration.put bucket=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3",
        operation="put_object_lock_configuration",
        op_kwargs={
            "Bucket": bucket,
            "ObjectLockConfiguration": object_lock_configuration,
            "RequestPayer": request_payer,
            "Token": token,
            "ContentMD5": content_md5,
            "ExpectedBucketOwner": expected_bucket_owner,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def delete_multiple(hub, ctx, voice_connector_id: str, usernames: List) -> None:
    r"""
    **Autogenerated function**

    Deletes the specified SIP credentials used by your equipment to authenticate during call termination.

    Args:
        voice_connector_id(str): The Amazon Chime Voice Connector ID.
        usernames(List): The RFC2617 compliant username associated with the SIP credentials, in US-ASCII format.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.voice_connector.termination.credential.delete_multiple(
                ctx, voice_connector_id=value, usernames=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.voice_connector.termination.credential.delete_multiple voice_connector_id=value, usernames=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="delete_voice_connector_termination_credentials",
        op_kwargs={"VoiceConnectorId": voice_connector_id, "Usernames": usernames},
    )


async def list_all(hub, ctx, voice_connector_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Lists the SIP credentials for the specified Amazon Chime Voice Connector.

    Args:
        voice_connector_id(str): The Amazon Chime Voice Connector ID.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.voice_connector.termination.credential.list_all(
                ctx, voice_connector_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.voice_connector.termination.credential.list_all voice_connector_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="list_voice_connector_termination_credentials",
        op_kwargs={"VoiceConnectorId": voice_connector_id},
    )


async def put_multiple(
    hub, ctx, voice_connector_id: str, credentials: List = None
) -> None:
    r"""
    **Autogenerated function**

    Adds termination SIP credentials for the specified Amazon Chime Voice Connector.

    Args:
        voice_connector_id(str): The Amazon Chime Voice Connector ID.
        credentials(List, optional): The termination SIP credentials. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.voice_connector.termination.credential.put_multiple(
                ctx, voice_connector_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.voice_connector.termination.credential.put_multiple voice_connector_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="put_voice_connector_termination_credentials",
        op_kwargs={"VoiceConnectorId": voice_connector_id, "Credentials": credentials},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def detect(hub, ctx, text: str) -> Dict:
    r"""
    **Autogenerated function**

     Inspects the clinical text for protected health information (PHI) entities and returns the entity category, location, and confidence score for each entity. Amazon Comprehend Medical only detects entities in English language texts.

    Args:
        text(str):  A UTF-8 text string containing the clinical content being examined for PHI entities. Each string must contain fewer than 20,000 bytes of characters.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.comprehendmedical.phi.init.detect(ctx, text=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.comprehendmedical.phi.init.detect text=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="comprehendmedical",
        operation="detect_phi",
        op_kwargs={"Text": text},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def delete(
    hub,
    ctx,
    database_name: str,
    table_name: str,
    column_name: str,
    catalog_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves table statistics of columns. The Identity and Access Management (IAM) permission required for this operation is DeleteTable.

    Args:
        catalog_id(str, optional): The ID of the Data Catalog where the partitions in question reside. If none is supplied, the Amazon Web Services account ID is used by default. Defaults to None.
        database_name(str): The name of the catalog database where the partitions reside.
        table_name(str): The name of the partitions' table.
        column_name(str): The name of the column.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.column_statistic_for_.table.delete(
                ctx, database_name=value, table_name=value, column_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.column_statistic_for_.table.delete database_name=value, table_name=value, column_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="delete_column_statistics_for_table",
        op_kwargs={
            "CatalogId": catalog_id,
            "DatabaseName": database_name,
            "TableName": table_name,
            "ColumnName": column_name,
        },
    )


async def get(
    hub,
    ctx,
    database_name: str,
    table_name: str,
    column_names: List,
    catalog_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves table statistics of columns. The Identity and Access Management (IAM) permission required for this operation is GetTable.

    Args:
        catalog_id(str, optional): The ID of the Data Catalog where the partitions in question reside. If none is supplied, the Amazon Web Services account ID is used by default. Defaults to None.
        database_name(str): The name of the catalog database where the partitions reside.
        table_name(str): The name of the partitions' table.
        column_names(List): A list of the column names.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.column_statistic_for_.table.get(
                ctx, database_name=value, table_name=value, column_names=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.column_statistic_for_.table.get database_name=value, table_name=value, column_names=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="get_column_statistics_for_table",
        op_kwargs={
            "CatalogId": catalog_id,
            "DatabaseName": database_name,
            "TableName": table_name,
            "ColumnNames": column_names,
        },
    )


async def update(
    hub,
    ctx,
    database_name: str,
    table_name: str,
    column_statistics_list: List,
    catalog_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates or updates table statistics of columns. The Identity and Access Management (IAM) permission required for this operation is UpdateTable.

    Args:
        catalog_id(str, optional): The ID of the Data Catalog where the partitions in question reside. If none is supplied, the Amazon Web Services account ID is used by default. Defaults to None.
        database_name(str): The name of the catalog database where the partitions reside.
        table_name(str): The name of the partitions' table.
        column_statistics_list(List): A list of the column statistics.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.glue.column_statistic_for_.table.update(
                ctx, database_name=value, table_name=value, column_statistics_list=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.glue.column_statistic_for_.table.update database_name=value, table_name=value, column_statistics_list=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="glue",
        operation="update_column_statistics_for_table",
        op_kwargs={
            "CatalogId": catalog_id,
            "DatabaseName": database_name,
            "TableName": table_name,
            "ColumnStatisticsList": column_statistics_list,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def admin(
    hub,
    ctx,
    user_pool_id: str,
    client_id: str,
    challenge_name: str,
    challenge_responses: Dict = None,
    session: str = None,
    analytics_metadata: Dict = None,
    context_data: Dict = None,
    client_metadata: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Responds to an authentication challenge, as an administrator.  This action might generate an SMS text message. Starting June 1, 2021, U.S. telecom carriers require that you register an origination phone number before you can send SMS messages to U.S. phone numbers. If you use SMS text messages in Amazon Cognito, you must register a phone number with Amazon Pinpoint. Cognito will use the the registered number automatically. Otherwise, Cognito users that must receive SMS messages might be unable to sign up, activate their accounts, or sign in. If you have never used SMS text messages with Amazon Cognito or any other Amazon Web Service, Amazon SNS might place your account in SMS sandbox. In  sandbox mode , you’ll have limitations, such as sending messages to only verified phone numbers. After testing in the sandbox environment, you can move out of the SMS sandbox and into production. For more information, see  SMS message settings for Cognito User Pools in the Amazon Cognito Developer Guide.   Calling this action requires developer credentials.

    Args:
        user_pool_id(str): The ID of the Amazon Cognito user pool.
        client_id(str): The app client ID.
        challenge_name(str): The challenge name. For more information, see AdminInitiateAuth.
        challenge_responses(Dict, optional): The challenge responses. These are inputs corresponding to the value of ChallengeName, for example:    SMS_MFA: SMS_MFA_CODE, USERNAME, SECRET_HASH (if app client is configured with client secret).    PASSWORD_VERIFIER: PASSWORD_CLAIM_SIGNATURE, PASSWORD_CLAIM_SECRET_BLOCK, TIMESTAMP, USERNAME, SECRET_HASH (if app client is configured with client secret).    ADMIN_NO_SRP_AUTH: PASSWORD, USERNAME, SECRET_HASH (if app client is configured with client secret).     NEW_PASSWORD_REQUIRED: NEW_PASSWORD, any other required attributes, USERNAME, SECRET_HASH (if app client is configured with client secret).     MFA_SETUP requires USERNAME, plus you need to use the session value returned by VerifySoftwareToken in the Session parameter.   The value of the USERNAME attribute must be the user's actual username, not an alias (such as email address or phone number). To make this easier, the AdminInitiateAuth response includes the actual username value in the USERNAMEUSER_ID_FOR_SRP attribute, even if you specified an alias in your call to AdminInitiateAuth. Defaults to None.
        session(str, optional): The session which should be passed both ways in challenge-response calls to the service. If InitiateAuth or RespondToAuthChallenge API call determines that the caller needs to go through another challenge, they return a session with other challenge parameters. This session should be passed as it is to the next RespondToAuthChallenge API call. Defaults to None.
        analytics_metadata(Dict, optional): The analytics metadata for collecting Amazon Pinpoint metrics for AdminRespondToAuthChallenge calls. Defaults to None.
        context_data(Dict, optional): Contextual data such as the user's device fingerprint, IP address, or location used for evaluating the risk of an unexpected event by Amazon Cognito advanced security. Defaults to None.
        client_metadata(Dict, optional): A map of custom key-value pairs that you can provide as input for any custom workflows that this action triggers.  You create custom workflows by assigning Lambda functions to user pool triggers. When you use the AdminRespondToAuthChallenge API action, Amazon Cognito invokes any functions that are assigned to the following triggers: pre sign-up, custom message, post authentication, user migration, pre token generation, define auth challenge, create auth challenge, and verify auth challenge response. When Amazon Cognito invokes any of these functions, it passes a JSON payload, which the function receives as input. This payload contains a clientMetadata attribute, which provides the data that you assigned to the ClientMetadata parameter in your AdminRespondToAuthChallenge request. In your function code in Lambda, you can process the clientMetadata value to enhance your workflow for your specific needs. For more information, see Customizing User Pool Workflows with Lambda Triggers in the Amazon Cognito Developer Guide.  Take the following limitations into consideration when you use the ClientMetadata parameter:   Amazon Cognito does not store the ClientMetadata value. This data is available only to Lambda triggers that are assigned to a user pool to support custom workflows. If your user pool configuration does not include triggers, the ClientMetadata parameter serves no purpose.   Amazon Cognito does not validate the ClientMetadata value.   Amazon Cognito does not encrypt the the ClientMetadata value, so don't use it to provide sensitive information.   . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_idp.respond_to_auth_challenge.admin(
                ctx, user_pool_id=value, client_id=value, challenge_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_idp.respond_to_auth_challenge.admin user_pool_id=value, client_id=value, challenge_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-idp",
        operation="admin_respond_to_auth_challenge",
        op_kwargs={
            "UserPoolId": user_pool_id,
            "ClientId": client_id,
            "ChallengeName": challenge_name,
            "ChallengeResponses": challenge_responses,
            "Session": session,
            "AnalyticsMetadata": analytics_metadata,
            "ContextData": context_data,
            "ClientMetadata": client_metadata,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, user_name: str, aws_account_id: str, namespace: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the Amazon QuickSight user that is associated with the identity of the Identity and Access Management (IAM) user or role that's making the call. The IAM user isn't deleted as a result of this call.

    Args:
        user_name(str): The name of the user that you want to delete.
        aws_account_id(str): The ID for the Amazon Web Services account; that the user is in. Currently, you use the ID for the Amazon Web Services account; that contains your Amazon QuickSight account.
        namespace(str): The namespace. Currently, you should set this to default.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.quicksight.user.init.delete(
                ctx, user_name=value, aws_account_id=value, namespace=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.quicksight.user.init.delete user_name=value, aws_account_id=value, namespace=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="quicksight",
        operation="delete_user",
        op_kwargs={
            "UserName": user_name,
            "AwsAccountId": aws_account_id,
            "Namespace": namespace,
        },
    )


async def describe(
    hub, ctx, user_name: str, aws_account_id: str, namespace: str
) -> Dict:
    r"""
    **Autogenerated function**

    Returns information about a user, given the user name.

    Args:
        user_name(str): The name of the user that you want to describe.
        aws_account_id(str): The ID for the Amazon Web Services account; that the user is in. Currently, you use the ID for the Amazon Web Services account; that contains your Amazon QuickSight account.
        namespace(str): The namespace. Currently, you should set this to default.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.quicksight.user.init.describe(
                ctx, user_name=value, aws_account_id=value, namespace=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.quicksight.user.init.describe user_name=value, aws_account_id=value, namespace=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="quicksight",
        operation="describe_user",
        op_kwargs={
            "UserName": user_name,
            "AwsAccountId": aws_account_id,
            "Namespace": namespace,
        },
    )


async def list_all(
    hub,
    ctx,
    aws_account_id: str,
    namespace: str,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of all of the Amazon QuickSight users belonging to this account.

    Args:
        aws_account_id(str): The ID for the Amazon Web Services account; that the user is in. Currently, you use the ID for the Amazon Web Services account; that contains your Amazon QuickSight account.
        next_token(str, optional): A pagination token that can be used in a subsequent request. Defaults to None.
        max_results(int, optional): The maximum number of results to return from this request. Defaults to None.
        namespace(str): The namespace. Currently, you should set this to default.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.quicksight.user.init.list_all(
                ctx, aws_account_id=value, namespace=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.quicksight.user.init.list_all aws_account_id=value, namespace=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="quicksight",
        operation="list_users",
        op_kwargs={
            "AwsAccountId": aws_account_id,
            "NextToken": next_token,
            "MaxResults": max_results,
            "Namespace": namespace,
        },
    )


async def register(
    hub,
    ctx,
    identity_type: str,
    email: str,
    user_role: str,
    aws_account_id: str,
    namespace: str,
    iam_arn: str = None,
    session_name: str = None,
    user_name: str = None,
    custom_permissions_name: str = None,
    external_login_federation_provider_type: str = None,
    custom_federation_provider_url: str = None,
    external_login_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates an Amazon QuickSight user, whose identity is associated with the AWS Identity and Access Management (IAM) identity or role specified in the request.

    Args:
        identity_type(str): Amazon QuickSight supports several ways of managing the identity of users. This parameter accepts two values:    IAM: A user whose identity maps to an existing IAM user or role.     QUICKSIGHT: A user whose identity is owned and managed internally by Amazon QuickSight.   .
        email(str): The email address of the user that you want to register.
        user_role(str): The Amazon QuickSight role for the user. The user role can be one of the following:    READER: A user who has read-only access to dashboards.    AUTHOR: A user who can create data sources, datasets, analyses, and dashboards.    ADMIN: A user who is an author, who can also manage Amazon QuickSight settings.    RESTRICTED_READER: This role isn't currently available for use.    RESTRICTED_AUTHOR: This role isn't currently available for use.  .
        iam_arn(str, optional): The ARN of the IAM user or role that you are registering with Amazon QuickSight. . Defaults to None.
        session_name(str, optional): You need to use this parameter only when you register one or more users using an assumed IAM role. You don't need to provide the session name for other scenarios, for example when you are registering an IAM user or an Amazon QuickSight user. You can register multiple users using the same IAM role if each user has a different session name. For more information on assuming IAM roles, see  assume-role  in the AWS CLI Reference. . Defaults to None.
        aws_account_id(str): The ID for the Amazon Web Services account; that the user is in. Currently, you use the ID for the Amazon Web Services account; that contains your Amazon QuickSight account.
        namespace(str): The namespace. Currently, you should set this to default.
        user_name(str, optional): The Amazon QuickSight user name that you want to create for the user you are registering. Defaults to None.
        custom_permissions_name(str, optional): (Enterprise edition only) The name of the custom permissions profile that you want to assign to this user. Customized permissions allows you to control a user's access by restricting access the following operations:   Create and update data sources   Create and update datasets   Create and update email reports   Subscribe to email reports   To add custom permissions to an existing user, use  UpdateUser  instead. A set of custom permissions includes any combination of these restrictions. Currently, you need to create the profile names for custom permission sets by using the QuickSight console. Then, you use the RegisterUser API operation to assign the named set of permissions to a QuickSight user.  QuickSight custom permissions are applied through IAM policies. Therefore, they override the permissions typically granted by assigning QuickSight users to one of the default security cohorts in QuickSight (admin, author, reader). This feature is available only to QuickSight Enterprise edition subscriptions. Defaults to None.
        external_login_federation_provider_type(str, optional): The type of supported external login provider that provides identity to let a user federate into Amazon QuickSight with an associated AWS Identity and Access Management (IAM) role. The type of supported external login provider can be one of the following.    COGNITO: Amazon Cognito. The provider URL is cognito-identity.amazonaws.com. When choosing the COGNITO provider type, don’t use the "CustomFederationProviderUrl" parameter which is only needed when the external provider is custom.    CUSTOM_OIDC: Custom OpenID Connect (OIDC) provider. When choosing CUSTOM_OIDC type, use the CustomFederationProviderUrl parameter to provide the custom OIDC provider URL.  . Defaults to None.
        custom_federation_provider_url(str, optional): The URL of the custom OpenID Connect (OIDC) provider that provides identity to let a user federate into QuickSight with an associated AWS Identity and Access Management (IAM) role. This parameter should only be used when ExternalLoginFederationProviderType parameter is set to CUSTOM_OIDC. Defaults to None.
        external_login_id(str, optional): The identity ID for a user in the external login provider. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.quicksight.user.init.register(
                ctx,
                identity_type=value,
                email=value,
                user_role=value,
                aws_account_id=value,
                namespace=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.quicksight.user.init.register identity_type=value, email=value, user_role=value, aws_account_id=value, namespace=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="quicksight",
        operation="register_user",
        op_kwargs={
            "IdentityType": identity_type,
            "Email": email,
            "UserRole": user_role,
            "IamArn": iam_arn,
            "SessionName": session_name,
            "AwsAccountId": aws_account_id,
            "Namespace": namespace,
            "UserName": user_name,
            "CustomPermissionsName": custom_permissions_name,
            "ExternalLoginFederationProviderType": external_login_federation_provider_type,
            "CustomFederationProviderUrl": custom_federation_provider_url,
            "ExternalLoginId": external_login_id,
        },
    )


async def update(
    hub,
    ctx,
    user_name: str,
    aws_account_id: str,
    namespace: str,
    email: str,
    role: str,
    custom_permissions_name: str = None,
    unapply_custom_permissions: bool = None,
    external_login_federation_provider_type: str = None,
    custom_federation_provider_url: str = None,
    external_login_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates an Amazon QuickSight user.

    Args:
        user_name(str): The Amazon QuickSight user name that you want to update.
        aws_account_id(str): The ID for the Amazon Web Services account; that the user is in. Currently, you use the ID for the Amazon Web Services account; that contains your Amazon QuickSight account.
        namespace(str): The namespace. Currently, you should set this to default.
        email(str): The email address of the user that you want to update.
        role(str): The Amazon QuickSight role of the user. The role can be one of the following default security cohorts:    READER: A user who has read-only access to dashboards.    AUTHOR: A user who can create data sources, datasets, analyses, and dashboards.    ADMIN: A user who is an author, who can also manage Amazon QuickSight settings.   The name of the QuickSight role is invisible to the user except for the console screens dealing with permissions.
        custom_permissions_name(str, optional): (Enterprise edition only) The name of the custom permissions profile that you want to assign to this user. Customized permissions allows you to control a user's access by restricting access the following operations:   Create and update data sources   Create and update datasets   Create and update email reports   Subscribe to email reports   A set of custom permissions includes any combination of these restrictions. Currently, you need to create the profile names for custom permission sets by using the QuickSight console. Then, you use the RegisterUser API operation to assign the named set of permissions to a QuickSight user.  QuickSight custom permissions are applied through IAM policies. Therefore, they override the permissions typically granted by assigning QuickSight users to one of the default security cohorts in QuickSight (admin, author, reader). This feature is available only to QuickSight Enterprise edition subscriptions. Defaults to None.
        unapply_custom_permissions(bool, optional): A flag that you use to indicate that you want to remove all custom permissions from this user. Using this parameter resets the user to the state it was in before a custom permissions profile was applied. This parameter defaults to NULL and it doesn't accept any other value. Defaults to None.
        external_login_federation_provider_type(str, optional): The type of supported external login provider that provides identity to let a user federate into QuickSight with an associated AWS Identity and Access Management (IAM) role. The type of supported external login provider can be one of the following.    COGNITO: Amazon Cognito. The provider URL is cognito-identity.amazonaws.com. When choosing the COGNITO provider type, don’t use the "CustomFederationProviderUrl" parameter which is only needed when the external provider is custom.    CUSTOM_OIDC: Custom OpenID Connect (OIDC) provider. When choosing CUSTOM_OIDC type, use the CustomFederationProviderUrl parameter to provide the custom OIDC provider URL.    NONE: This clears all the previously saved external login information for a user. Use  DescribeUser  API to check the external login information.  . Defaults to None.
        custom_federation_provider_url(str, optional): The URL of the custom OpenID Connect (OIDC) provider that provides identity to let a user federate into QuickSight with an associated AWS Identity and Access Management (IAM) role. This parameter should only be used when ExternalLoginFederationProviderType parameter is set to CUSTOM_OIDC. Defaults to None.
        external_login_id(str, optional): The identity ID for a user in the external login provider. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.quicksight.user.init.update(
                ctx, user_name=value, aws_account_id=value, namespace=value, email=value, role=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.quicksight.user.init.update user_name=value, aws_account_id=value, namespace=value, email=value, role=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="quicksight",
        operation="update_user",
        op_kwargs={
            "UserName": user_name,
            "AwsAccountId": aws_account_id,
            "Namespace": namespace,
            "Email": email,
            "Role": role,
            "CustomPermissionsName": custom_permissions_name,
            "UnapplyCustomPermissions": unapply_custom_permissions,
            "ExternalLoginFederationProviderType": external_login_federation_provider_type,
            "CustomFederationProviderUrl": custom_federation_provider_url,
            "ExternalLoginId": external_login_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    project_id: str,
    dashboard_name: str,
    dashboard_definition: str,
    dashboard_description: str = None,
    client_token: str = None,
    tags: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a dashboard in an IoT SiteWise Monitor project.

    Args:
        project_id(str): The ID of the project in which to create the dashboard.
        dashboard_name(str): A friendly name for the dashboard.
        dashboard_description(str, optional): A description for the dashboard. Defaults to None.
        dashboard_definition(str): The dashboard definition specified in a JSON literal. For detailed information, see Creating dashboards (CLI) in the IoT SiteWise User Guide.
        client_token(str, optional): A unique case-sensitive identifier that you can provide to ensure the idempotency of the request. Don't reuse this client token if a new idempotent request is required. Defaults to None.
        tags(Dict, optional): A list of key-value pairs that contain metadata for the dashboard. For more information, see Tagging your IoT SiteWise resources in the IoT SiteWise User Guide. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.dashboard.create(
                ctx, project_id=value, dashboard_name=value, dashboard_definition=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.dashboard.create project_id=value, dashboard_name=value, dashboard_definition=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="create_dashboard",
        op_kwargs={
            "projectId": project_id,
            "dashboardName": dashboard_name,
            "dashboardDescription": dashboard_description,
            "dashboardDefinition": dashboard_definition,
            "clientToken": client_token,
            "tags": tags,
        },
    )


async def delete(hub, ctx, dashboard_id: str, client_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a dashboard from IoT SiteWise Monitor.

    Args:
        dashboard_id(str): The ID of the dashboard to delete.
        client_token(str, optional): A unique case-sensitive identifier that you can provide to ensure the idempotency of the request. Don't reuse this client token if a new idempotent request is required. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.dashboard.delete(ctx, dashboard_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.dashboard.delete dashboard_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="delete_dashboard",
        op_kwargs={"dashboardId": dashboard_id, "clientToken": client_token},
    )


async def describe(hub, ctx, dashboard_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves information about a dashboard.

    Args:
        dashboard_id(str): The ID of the dashboard.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.dashboard.describe(ctx, dashboard_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.dashboard.describe dashboard_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="describe_dashboard",
        op_kwargs={"dashboardId": dashboard_id},
    )


async def list_all(
    hub, ctx, project_id: str, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves a paginated list of dashboards for an IoT SiteWise Monitor project.

    Args:
        project_id(str): The ID of the project.
        next_token(str, optional): The token to be used for the next set of paginated results. Defaults to None.
        max_results(int, optional): The maximum number of results to return for each paginated request. Default: 50. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.dashboard.list_all(ctx, project_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.dashboard.list_all project_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="list_dashboards",
        op_kwargs={
            "projectId": project_id,
            "nextToken": next_token,
            "maxResults": max_results,
        },
    )


async def update(
    hub,
    ctx,
    dashboard_id: str,
    dashboard_name: str,
    dashboard_definition: str,
    dashboard_description: str = None,
    client_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates an IoT SiteWise Monitor dashboard.

    Args:
        dashboard_id(str): The ID of the dashboard to update.
        dashboard_name(str): A new friendly name for the dashboard.
        dashboard_description(str, optional): A new description for the dashboard. Defaults to None.
        dashboard_definition(str): The new dashboard definition, as specified in a JSON literal. For detailed information, see Creating dashboards (CLI) in the IoT SiteWise User Guide.
        client_token(str, optional): A unique case-sensitive identifier that you can provide to ensure the idempotency of the request. Don't reuse this client token if a new idempotent request is required. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.dashboard.update(
                ctx, dashboard_id=value, dashboard_name=value, dashboard_definition=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.dashboard.update dashboard_id=value, dashboard_name=value, dashboard_definition=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="update_dashboard",
        op_kwargs={
            "dashboardId": dashboard_id,
            "dashboardName": dashboard_name,
            "dashboardDescription": dashboard_description,
            "dashboardDefinition": dashboard_definition,
            "clientToken": client_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    api_id: str,
    route_key: str,
    api_key_required: bool = None,
    authorization_scopes: List = None,
    authorization_type: str = None,
    authorizer_id: str = None,
    model_selection_expression: str = None,
    operation_name: str = None,
    request_models: Dict = None,
    request_parameters: Dict = None,
    route_response_selection_expression: str = None,
    target: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a Route for an API.

    Args:
        api_id(str): The API identifier.
        api_key_required(bool, optional): Specifies whether an API key is required for the route. Supported only for WebSocket APIs. Defaults to None.
        authorization_scopes(List, optional): The authorization scopes supported by this route. Defaults to None.
        authorization_type(str, optional): The authorization type for the route. For WebSocket APIs, valid values are NONE for open access, AWS_IAM for using AWS IAM permissions, and CUSTOM for using a Lambda authorizer For HTTP APIs, valid values are NONE for open access, JWT for using JSON Web Tokens, AWS_IAM for using AWS IAM permissions, and CUSTOM for using a Lambda authorizer. Defaults to None.
        authorizer_id(str, optional): The identifier of the Authorizer resource to be associated with this route. The authorizer identifier is generated by API Gateway when you created the authorizer. Defaults to None.
        model_selection_expression(str, optional): The model selection expression for the route. Supported only for WebSocket APIs. Defaults to None.
        operation_name(str, optional): The operation name for the route. Defaults to None.
        request_models(Dict, optional): The request models for the route. Supported only for WebSocket APIs. Defaults to None.
        request_parameters(Dict, optional): The request parameters for the route. Supported only for WebSocket APIs. Defaults to None.
        route_key(str): The route key for the route.
        route_response_selection_expression(str, optional): The route response selection expression for the route. Supported only for WebSocket APIs. Defaults to None.
        target(str, optional): The target for the route. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.route.init.create(
                ctx, api_id=value, route_key=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.route.init.create api_id=value, route_key=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="create_route",
        op_kwargs={
            "ApiId": api_id,
            "ApiKeyRequired": api_key_required,
            "AuthorizationScopes": authorization_scopes,
            "AuthorizationType": authorization_type,
            "AuthorizerId": authorizer_id,
            "ModelSelectionExpression": model_selection_expression,
            "OperationName": operation_name,
            "RequestModels": request_models,
            "RequestParameters": request_parameters,
            "RouteKey": route_key,
            "RouteResponseSelectionExpression": route_response_selection_expression,
            "Target": target,
        },
    )


async def delete(hub, ctx, api_id: str, route_id: str) -> None:
    r"""
    **Autogenerated function**

    Deletes a Route.

    Args:
        api_id(str): The API identifier.
        route_id(str): The route ID.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.route.init.delete(
                ctx, api_id=value, route_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.route.init.delete api_id=value, route_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="delete_route",
        op_kwargs={"ApiId": api_id, "RouteId": route_id},
    )


async def get(hub, ctx, api_id: str, route_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Gets a Route.

    Args:
        api_id(str): The API identifier.
        route_id(str): The route ID.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.route.init.get(ctx, api_id=value, route_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.route.init.get api_id=value, route_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="get_route",
        op_kwargs={"ApiId": api_id, "RouteId": route_id},
    )


async def get_all(
    hub, ctx, api_id: str, max_results: str = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Gets the Routes for an API.

    Args:
        api_id(str): The API identifier.
        max_results(str, optional): The maximum number of elements to be returned for this resource. Defaults to None.
        next_token(str, optional): The next page of elements from this collection. Not valid for the last element of the collection. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.route.init.get_all(ctx, api_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.route.init.get_all api_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="get_routes",
        op_kwargs={"ApiId": api_id, "MaxResults": max_results, "NextToken": next_token},
    )


async def update(
    hub,
    ctx,
    api_id: str,
    route_id: str,
    api_key_required: bool = None,
    authorization_scopes: List = None,
    authorization_type: str = None,
    authorizer_id: str = None,
    model_selection_expression: str = None,
    operation_name: str = None,
    request_models: Dict = None,
    request_parameters: Dict = None,
    route_key: str = None,
    route_response_selection_expression: str = None,
    target: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates a Route.

    Args:
        api_id(str): The API identifier.
        api_key_required(bool, optional): Specifies whether an API key is required for the route. Supported only for WebSocket APIs. Defaults to None.
        authorization_scopes(List, optional): The authorization scopes supported by this route. Defaults to None.
        authorization_type(str, optional): The authorization type for the route. For WebSocket APIs, valid values are NONE for open access, AWS_IAM for using AWS IAM permissions, and CUSTOM for using a Lambda authorizer For HTTP APIs, valid values are NONE for open access, JWT for using JSON Web Tokens, AWS_IAM for using AWS IAM permissions, and CUSTOM for using a Lambda authorizer. Defaults to None.
        authorizer_id(str, optional): The identifier of the Authorizer resource to be associated with this route. The authorizer identifier is generated by API Gateway when you created the authorizer. Defaults to None.
        model_selection_expression(str, optional): The model selection expression for the route. Supported only for WebSocket APIs. Defaults to None.
        operation_name(str, optional): The operation name for the route. Defaults to None.
        request_models(Dict, optional): The request models for the route. Supported only for WebSocket APIs. Defaults to None.
        request_parameters(Dict, optional): The request parameters for the route. Supported only for WebSocket APIs. Defaults to None.
        route_id(str): The route ID.
        route_key(str, optional): The route key for the route. Defaults to None.
        route_response_selection_expression(str, optional): The route response selection expression for the route. Supported only for WebSocket APIs. Defaults to None.
        target(str, optional): The target for the route. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.apigatewayv2.route.init.update(
                ctx, api_id=value, route_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.apigatewayv2.route.init.update api_id=value, route_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="apigatewayv2",
        operation="update_route",
        op_kwargs={
            "ApiId": api_id,
            "ApiKeyRequired": api_key_required,
            "AuthorizationScopes": authorization_scopes,
            "AuthorizationType": authorization_type,
            "AuthorizerId": authorizer_id,
            "ModelSelectionExpression": model_selection_expression,
            "OperationName": operation_name,
            "RequestModels": request_models,
            "RequestParameters": request_parameters,
            "RouteId": route_id,
            "RouteKey": route_key,
            "RouteResponseSelectionExpression": route_response_selection_expression,
            "Target": target,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"import_": "import"}


async def delete(
    hub, ctx, server_id: str, ssh_public_key_id: str, user_name: str
) -> None:
    r"""
    **Autogenerated function**

    Deletes a user's Secure Shell (SSH) public key. No response is returned from this operation.

    Args:
        server_id(str): A system-assigned unique identifier for a file transfer protocol-enabled server instance that has the user assigned to it.
        ssh_public_key_id(str): A unique identifier used to reference your user's specific SSH key.
        user_name(str): A unique string that identifies a user whose public key is being deleted.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.transfer.ssh_public_key.delete(ctx, server_id=value, ssh_public_key_id=value, user_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.transfer.ssh_public_key.delete server_id=value, ssh_public_key_id=value, user_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="transfer",
        operation="delete_ssh_public_key",
        op_kwargs={
            "ServerId": server_id,
            "SshPublicKeyId": ssh_public_key_id,
            "UserName": user_name,
        },
    )


async def import_(
    hub, ctx, server_id: str, ssh_public_key_body: str, user_name: str
) -> Dict:
    r"""
    **Autogenerated function**

    Adds a Secure Shell (SSH) public key to a user account identified by a UserName value assigned to the specific file transfer protocol-enabled server, identified by ServerId. The response returns the UserName value, the ServerId value, and the name of the SshPublicKeyId.

    Args:
        server_id(str): A system-assigned unique identifier for a server.
        ssh_public_key_body(str): The public key portion of an SSH key pair.
        user_name(str): The name of the user account that is assigned to one or more servers.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.transfer.ssh_public_key.import(ctx, server_id=value, ssh_public_key_body=value, user_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.transfer.ssh_public_key.import server_id=value, ssh_public_key_body=value, user_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="transfer",
        operation="import_ssh_public_key",
        op_kwargs={
            "ServerId": server_id,
            "SshPublicKeyBody": ssh_public_key_body,
            "UserName": user_name,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def delete(
    hub,
    ctx,
    policy_name: str,
    service_namespace: str,
    resource_id: str,
    scalable_dimension: str,
) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified scaling policy for an Application Auto Scaling scalable target. Deleting a step scaling policy deletes the underlying alarm action, but does not delete the CloudWatch alarm associated with the scaling policy, even if it no longer has an associated action. For more information, see Delete a step scaling policy and Delete a target tracking scaling policy in the Application Auto Scaling User Guide.

    Args:
        policy_name(str): The name of the scaling policy.
        service_namespace(str): The namespace of the AWS service that provides the resource. For a resource provided by your own application or service, use custom-resource instead.
        resource_id(str): The identifier of the resource associated with the scalable target. This string consists of the resource type and unique identifier.   ECS service - The resource type is service and the unique identifier is the cluster name and service name. Example: service/default/sample-webapp.   Spot Fleet request - The resource type is spot-fleet-request and the unique identifier is the Spot Fleet request ID. Example: spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE.   EMR cluster - The resource type is instancegroup and the unique identifier is the cluster ID and instance group ID. Example: instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0.   AppStream 2.0 fleet - The resource type is fleet and the unique identifier is the fleet name. Example: fleet/sample-fleet.   DynamoDB table - The resource type is table and the unique identifier is the table name. Example: table/my-table.   DynamoDB global secondary index - The resource type is index and the unique identifier is the index name. Example: table/my-table/index/my-table-index.   Aurora DB cluster - The resource type is cluster and the unique identifier is the cluster name. Example: cluster:my-db-cluster.   Amazon SageMaker endpoint variant - The resource type is variant and the unique identifier is the resource ID. Example: endpoint/my-end-point/variant/KMeansClustering.   Custom resources are not supported with a resource type. This parameter must specify the OutputValue from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our GitHub repository.   Amazon Comprehend document classification endpoint - The resource type and unique identifier are specified using the endpoint ARN. Example: arn:aws:comprehend:us-west-2:123456789012:document-classifier-endpoint/EXAMPLE.   Amazon Comprehend entity recognizer endpoint - The resource type and unique identifier are specified using the endpoint ARN. Example: arn:aws:comprehend:us-west-2:123456789012:entity-recognizer-endpoint/EXAMPLE.   Lambda provisioned concurrency - The resource type is function and the unique identifier is the function name with a function version or alias name suffix that is not $LATEST. Example: function:my-function:prod or function:my-function:1.   Amazon Keyspaces table - The resource type is table and the unique identifier is the table name. Example: keyspace/mykeyspace/table/mytable.   Amazon MSK cluster - The resource type and unique identifier are specified using the cluster ARN. Example: arn:aws:kafka:us-east-1:123456789012:cluster/demo-cluster-1/6357e0b2-0e6a-4b86-a0b4-70df934c2e31-5.  .
        scalable_dimension(str): The scalable dimension. This string consists of the service namespace, resource type, and scaling property.    ecs:service:DesiredCount - The desired task count of an ECS service.    ec2:spot-fleet-request:TargetCapacity - The target capacity of a Spot Fleet request.    elasticmapreduce:instancegroup:InstanceCount - The instance count of an EMR Instance Group.    appstream:fleet:DesiredCapacity - The desired capacity of an AppStream 2.0 fleet.    dynamodb:table:ReadCapacityUnits - The provisioned read capacity for a DynamoDB table.    dynamodb:table:WriteCapacityUnits - The provisioned write capacity for a DynamoDB table.    dynamodb:index:ReadCapacityUnits - The provisioned read capacity for a DynamoDB global secondary index.    dynamodb:index:WriteCapacityUnits - The provisioned write capacity for a DynamoDB global secondary index.    rds:cluster:ReadReplicaCount - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition.    sagemaker:variant:DesiredInstanceCount - The number of EC2 instances for an Amazon SageMaker model endpoint variant.    custom-resource:ResourceType:Property - The scalable dimension for a custom resource provided by your own application or service.    comprehend:document-classifier-endpoint:DesiredInferenceUnits - The number of inference units for an Amazon Comprehend document classification endpoint.    comprehend:entity-recognizer-endpoint:DesiredInferenceUnits - The number of inference units for an Amazon Comprehend entity recognizer endpoint.    lambda:function:ProvisionedConcurrency - The provisioned concurrency for a Lambda function.    cassandra:table:ReadCapacityUnits - The provisioned read capacity for an Amazon Keyspaces table.    cassandra:table:WriteCapacityUnits - The provisioned write capacity for an Amazon Keyspaces table.    kafka:broker-storage:VolumeSize - The provisioned volume size (in GiB) for brokers in an Amazon MSK cluster.  .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.application_autoscaling.scaling.policy.delete(
                ctx,
                policy_name=value,
                service_namespace=value,
                resource_id=value,
                scalable_dimension=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.application_autoscaling.scaling.policy.delete policy_name=value, service_namespace=value, resource_id=value, scalable_dimension=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="application-autoscaling",
        operation="delete_scaling_policy",
        op_kwargs={
            "PolicyName": policy_name,
            "ServiceNamespace": service_namespace,
            "ResourceId": resource_id,
            "ScalableDimension": scalable_dimension,
        },
    )


async def describe_all(
    hub,
    ctx,
    service_namespace: str,
    policy_names: List = None,
    resource_id: str = None,
    scalable_dimension: str = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Describes the Application Auto Scaling scaling policies for the specified service namespace. You can filter the results using ResourceId, ScalableDimension, and PolicyNames. For more information, see Target tracking scaling policies and Step scaling policies in the Application Auto Scaling User Guide.

    Args:
        policy_names(List, optional): The names of the scaling policies to describe. Defaults to None.
        service_namespace(str): The namespace of the AWS service that provides the resource. For a resource provided by your own application or service, use custom-resource instead.
        resource_id(str, optional): The identifier of the resource associated with the scaling policy. This string consists of the resource type and unique identifier.   ECS service - The resource type is service and the unique identifier is the cluster name and service name. Example: service/default/sample-webapp.   Spot Fleet request - The resource type is spot-fleet-request and the unique identifier is the Spot Fleet request ID. Example: spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE.   EMR cluster - The resource type is instancegroup and the unique identifier is the cluster ID and instance group ID. Example: instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0.   AppStream 2.0 fleet - The resource type is fleet and the unique identifier is the fleet name. Example: fleet/sample-fleet.   DynamoDB table - The resource type is table and the unique identifier is the table name. Example: table/my-table.   DynamoDB global secondary index - The resource type is index and the unique identifier is the index name. Example: table/my-table/index/my-table-index.   Aurora DB cluster - The resource type is cluster and the unique identifier is the cluster name. Example: cluster:my-db-cluster.   Amazon SageMaker endpoint variant - The resource type is variant and the unique identifier is the resource ID. Example: endpoint/my-end-point/variant/KMeansClustering.   Custom resources are not supported with a resource type. This parameter must specify the OutputValue from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our GitHub repository.   Amazon Comprehend document classification endpoint - The resource type and unique identifier are specified using the endpoint ARN. Example: arn:aws:comprehend:us-west-2:123456789012:document-classifier-endpoint/EXAMPLE.   Amazon Comprehend entity recognizer endpoint - The resource type and unique identifier are specified using the endpoint ARN. Example: arn:aws:comprehend:us-west-2:123456789012:entity-recognizer-endpoint/EXAMPLE.   Lambda provisioned concurrency - The resource type is function and the unique identifier is the function name with a function version or alias name suffix that is not $LATEST. Example: function:my-function:prod or function:my-function:1.   Amazon Keyspaces table - The resource type is table and the unique identifier is the table name. Example: keyspace/mykeyspace/table/mytable.   Amazon MSK cluster - The resource type and unique identifier are specified using the cluster ARN. Example: arn:aws:kafka:us-east-1:123456789012:cluster/demo-cluster-1/6357e0b2-0e6a-4b86-a0b4-70df934c2e31-5.  . Defaults to None.
        scalable_dimension(str, optional): The scalable dimension. This string consists of the service namespace, resource type, and scaling property. If you specify a scalable dimension, you must also specify a resource ID.    ecs:service:DesiredCount - The desired task count of an ECS service.    ec2:spot-fleet-request:TargetCapacity - The target capacity of a Spot Fleet request.    elasticmapreduce:instancegroup:InstanceCount - The instance count of an EMR Instance Group.    appstream:fleet:DesiredCapacity - The desired capacity of an AppStream 2.0 fleet.    dynamodb:table:ReadCapacityUnits - The provisioned read capacity for a DynamoDB table.    dynamodb:table:WriteCapacityUnits - The provisioned write capacity for a DynamoDB table.    dynamodb:index:ReadCapacityUnits - The provisioned read capacity for a DynamoDB global secondary index.    dynamodb:index:WriteCapacityUnits - The provisioned write capacity for a DynamoDB global secondary index.    rds:cluster:ReadReplicaCount - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition.    sagemaker:variant:DesiredInstanceCount - The number of EC2 instances for an Amazon SageMaker model endpoint variant.    custom-resource:ResourceType:Property - The scalable dimension for a custom resource provided by your own application or service.    comprehend:document-classifier-endpoint:DesiredInferenceUnits - The number of inference units for an Amazon Comprehend document classification endpoint.    comprehend:entity-recognizer-endpoint:DesiredInferenceUnits - The number of inference units for an Amazon Comprehend entity recognizer endpoint.    lambda:function:ProvisionedConcurrency - The provisioned concurrency for a Lambda function.    cassandra:table:ReadCapacityUnits - The provisioned read capacity for an Amazon Keyspaces table.    cassandra:table:WriteCapacityUnits - The provisioned write capacity for an Amazon Keyspaces table.    kafka:broker-storage:VolumeSize - The provisioned volume size (in GiB) for brokers in an Amazon MSK cluster.  . Defaults to None.
        max_results(int, optional): The maximum number of scalable targets. This value can be between 1 and 50. The default value is 50. If this parameter is used, the operation returns up to MaxResults results at a time, along with a NextToken value. To get the next set of results, include the NextToken value in a subsequent call. If this parameter is not used, the operation returns up to 50 results and a NextToken value, if applicable. Defaults to None.
        next_token(str, optional): The token for the next set of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.application_autoscaling.scaling.policy.describe_all(
                ctx, service_namespace=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.application_autoscaling.scaling.policy.describe_all service_namespace=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="application-autoscaling",
        operation="describe_scaling_policies",
        op_kwargs={
            "PolicyNames": policy_names,
            "ServiceNamespace": service_namespace,
            "ResourceId": resource_id,
            "ScalableDimension": scalable_dimension,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )


async def put(
    hub,
    ctx,
    policy_name: str,
    service_namespace: str,
    resource_id: str,
    scalable_dimension: str,
    policy_type: str = None,
    step_scaling_policy_configuration: Dict = None,
    target_tracking_scaling_policy_configuration: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates or updates a scaling policy for an Application Auto Scaling scalable target. Each scalable target is identified by a service namespace, resource ID, and scalable dimension. A scaling policy applies to the scalable target identified by those three attributes. You cannot create a scaling policy until you have registered the resource as a scalable target. Multiple scaling policies can be in force at the same time for the same scalable target. You can have one or more target tracking scaling policies, one or more step scaling policies, or both. However, there is a chance that multiple policies could conflict, instructing the scalable target to scale out or in at the same time. Application Auto Scaling gives precedence to the policy that provides the largest capacity for both scale out and scale in. For example, if one policy increases capacity by 3, another policy increases capacity by 200 percent, and the current capacity is 10, Application Auto Scaling uses the policy with the highest calculated capacity (200% of 10 = 20) and scales out to 30.  We recommend caution, however, when using target tracking scaling policies with step scaling policies because conflicts between these policies can cause undesirable behavior. For example, if the step scaling policy initiates a scale-in activity before the target tracking policy is ready to scale in, the scale-in activity will not be blocked. After the scale-in activity completes, the target tracking policy could instruct the scalable target to scale out again.  For more information, see Target tracking scaling policies and Step scaling policies in the Application Auto Scaling User Guide.  If a scalable target is deregistered, the scalable target is no longer available to execute scaling policies. Any scaling policies that were specified for the scalable target are deleted.

    Args:
        policy_name(str): The name of the scaling policy.
        service_namespace(str): The namespace of the AWS service that provides the resource. For a resource provided by your own application or service, use custom-resource instead.
        resource_id(str): The identifier of the resource associated with the scaling policy. This string consists of the resource type and unique identifier.   ECS service - The resource type is service and the unique identifier is the cluster name and service name. Example: service/default/sample-webapp.   Spot Fleet request - The resource type is spot-fleet-request and the unique identifier is the Spot Fleet request ID. Example: spot-fleet-request/sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE.   EMR cluster - The resource type is instancegroup and the unique identifier is the cluster ID and instance group ID. Example: instancegroup/j-2EEZNYKUA1NTV/ig-1791Y4E1L8YI0.   AppStream 2.0 fleet - The resource type is fleet and the unique identifier is the fleet name. Example: fleet/sample-fleet.   DynamoDB table - The resource type is table and the unique identifier is the table name. Example: table/my-table.   DynamoDB global secondary index - The resource type is index and the unique identifier is the index name. Example: table/my-table/index/my-table-index.   Aurora DB cluster - The resource type is cluster and the unique identifier is the cluster name. Example: cluster:my-db-cluster.   Amazon SageMaker endpoint variant - The resource type is variant and the unique identifier is the resource ID. Example: endpoint/my-end-point/variant/KMeansClustering.   Custom resources are not supported with a resource type. This parameter must specify the OutputValue from the CloudFormation template stack used to access the resources. The unique identifier is defined by the service provider. More information is available in our GitHub repository.   Amazon Comprehend document classification endpoint - The resource type and unique identifier are specified using the endpoint ARN. Example: arn:aws:comprehend:us-west-2:123456789012:document-classifier-endpoint/EXAMPLE.   Amazon Comprehend entity recognizer endpoint - The resource type and unique identifier are specified using the endpoint ARN. Example: arn:aws:comprehend:us-west-2:123456789012:entity-recognizer-endpoint/EXAMPLE.   Lambda provisioned concurrency - The resource type is function and the unique identifier is the function name with a function version or alias name suffix that is not $LATEST. Example: function:my-function:prod or function:my-function:1.   Amazon Keyspaces table - The resource type is table and the unique identifier is the table name. Example: keyspace/mykeyspace/table/mytable.   Amazon MSK cluster - The resource type and unique identifier are specified using the cluster ARN. Example: arn:aws:kafka:us-east-1:123456789012:cluster/demo-cluster-1/6357e0b2-0e6a-4b86-a0b4-70df934c2e31-5.  .
        scalable_dimension(str): The scalable dimension. This string consists of the service namespace, resource type, and scaling property.    ecs:service:DesiredCount - The desired task count of an ECS service.    ec2:spot-fleet-request:TargetCapacity - The target capacity of a Spot Fleet request.    elasticmapreduce:instancegroup:InstanceCount - The instance count of an EMR Instance Group.    appstream:fleet:DesiredCapacity - The desired capacity of an AppStream 2.0 fleet.    dynamodb:table:ReadCapacityUnits - The provisioned read capacity for a DynamoDB table.    dynamodb:table:WriteCapacityUnits - The provisioned write capacity for a DynamoDB table.    dynamodb:index:ReadCapacityUnits - The provisioned read capacity for a DynamoDB global secondary index.    dynamodb:index:WriteCapacityUnits - The provisioned write capacity for a DynamoDB global secondary index.    rds:cluster:ReadReplicaCount - The count of Aurora Replicas in an Aurora DB cluster. Available for Aurora MySQL-compatible edition and Aurora PostgreSQL-compatible edition.    sagemaker:variant:DesiredInstanceCount - The number of EC2 instances for an Amazon SageMaker model endpoint variant.    custom-resource:ResourceType:Property - The scalable dimension for a custom resource provided by your own application or service.    comprehend:document-classifier-endpoint:DesiredInferenceUnits - The number of inference units for an Amazon Comprehend document classification endpoint.    comprehend:entity-recognizer-endpoint:DesiredInferenceUnits - The number of inference units for an Amazon Comprehend entity recognizer endpoint.    lambda:function:ProvisionedConcurrency - The provisioned concurrency for a Lambda function.    cassandra:table:ReadCapacityUnits - The provisioned read capacity for an Amazon Keyspaces table.    cassandra:table:WriteCapacityUnits - The provisioned write capacity for an Amazon Keyspaces table.    kafka:broker-storage:VolumeSize - The provisioned volume size (in GiB) for brokers in an Amazon MSK cluster.  .
        policy_type(str, optional): The policy type. This parameter is required if you are creating a scaling policy. The following policy types are supported:   TargetTrackingScaling—Not supported for Amazon EMR  StepScaling—Not supported for DynamoDB, Amazon Comprehend, Lambda, Amazon Keyspaces (for Apache Cassandra), or Amazon MSK. For more information, see Target tracking scaling policies and Step scaling policies in the Application Auto Scaling User Guide. Defaults to None.
        step_scaling_policy_configuration(Dict, optional): A step scaling policy. This parameter is required if you are creating a policy and the policy type is StepScaling. Defaults to None.
        target_tracking_scaling_policy_configuration(Dict, optional): A target tracking scaling policy. Includes support for predefined or customized metrics. This parameter is required if you are creating a policy and the policy type is TargetTrackingScaling. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.application_autoscaling.scaling.policy.put(
                ctx,
                policy_name=value,
                service_namespace=value,
                resource_id=value,
                scalable_dimension=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.application_autoscaling.scaling.policy.put policy_name=value, service_namespace=value, resource_id=value, scalable_dimension=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="application-autoscaling",
        operation="put_scaling_policy",
        op_kwargs={
            "PolicyName": policy_name,
            "ServiceNamespace": service_namespace,
            "ResourceId": resource_id,
            "ScalableDimension": scalable_dimension,
            "PolicyType": policy_type,
            "StepScalingPolicyConfiguration": step_scaling_policy_configuration,
            "TargetTrackingScalingPolicyConfiguration": target_tracking_scaling_policy_configuration,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""


__func_alias__ = {"set_": "set"}


async def set_(hub, ctx, policy_arn: str, version_id: str) -> None:
    r"""
    **Autogenerated function**

    Sets the specified version of the specified policy as the policy's default (operative) version. This operation affects all users, groups, and roles that the policy is attached to. To list the users, groups, and roles that the policy is attached to, use ListEntitiesForPolicy. For information about managed policies, see Managed policies and inline policies in the IAM User Guide.

    Args:
        policy_arn(str): The Amazon Resource Name (ARN) of the IAM policy whose default version you want to set. For more information about ARNs, see Amazon Resource Names (ARNs) in the Amazon Web Services General Reference.
        version_id(str): The version of the policy to set as the default (operative) version. For more information about managed policy versions, see Versioning for managed policies in the IAM User Guide.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.default_policy_version.set(
                ctx, policy_arn=value, version_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.default_policy_version.set policy_arn=value, version_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="set_default_policy_version",
        op_kwargs={"PolicyArn": policy_arn, "VersionId": version_id},
    )
